@@ -179,14 +179,16 @@ public:
 		true if one or more commands from this status interface client
 		are executing by Worker threads.
 	*/
-	bool IsBusy()
+	bool IsBusy() const
 	{ 
 		PWaitAndSignal lock(m_cmutex);
 		return m_numExecutingCommands > 0;
 	}
 
+	bool IsDone() const { return m_done; }
+
 	const PString& GetUser() const { return m_user; }
-	
+
 protected:
 	// override from class ServerSocket
 	virtual void Dispatch();
@@ -297,6 +299,8 @@ protected:
 	int m_instanceNo;
 	/// output trace level for this client
 	int m_traceLevel;
+	/// should the client be terminated, eg. after finishing a one-shot execute
+	bool m_done;
 
 	// vectors of regular expressions to be matched against Status messages
 	std::vector<PString> m_excludeFilterRegex;
@@ -368,7 +372,8 @@ SSHStatusClient::SSHStatusClient(int instanceNo) : StatusClient(instanceNo)
 
 SSHStatusClient::~SSHStatusClient()
 {
-    ssh_disconnect(m_session);
+	ssh_disconnect(m_session);
+	ssh_free(m_session);
 	if (m_sshbind) {
 		ssh_bind_set_fd(m_sshbind, -1);		// make sure StatusListener is not closed
 		ssh_bind_free(m_sshbind);
@@ -387,6 +392,9 @@ PBoolean SSHStatusClient::Accept(PSocket & socket)
 	m_session = ssh_new();
 
 	//ssh_bind_options_set(m_sshbind, SSH_BIND_OPTIONS_LOG_VERBOSITY_STR, "3");	// only enable for debugging
+	// disable compression, doesn't seem to work (bug in libssh)
+	ssh_options_set(m_session, SSH_OPTIONS_COMPRESSION_C_S, "none");
+	ssh_options_set(m_session, SSH_OPTIONS_COMPRESSION_S_C, "none");
 
 	bool keyAvailable = false;
 	PString dsakey = GkConfig()->GetString(authsec, "DSAKey", "/etc/ssh/ssh_host_dsa_key");
@@ -408,19 +416,19 @@ PBoolean SSHStatusClient::Accept(PSocket & socket)
 		return false;
 	}
 
-    if(ssh_init() < 0) {
+    if (ssh_init() < 0) {
 		PTRACE(1, "ssh_init() failed");
 		SNMP_TRAP(7, SNMPError, Network, "SSH init failed");
 		return false;
     }
 	ssh_bind_set_fd(m_sshbind, socket.GetHandle());
-	if(ssh_bind_accept(m_sshbind, m_session) == SSH_ERROR) {
+	if (ssh_bind_accept(m_sshbind, m_session) == SSH_ERROR) {
 		PTRACE(1, "ssh_bind_accept() failed: " << ssh_get_error(m_sshbind));
 		SNMP_TRAP(7, SNMPError, Network, "SSH bind failed");
 		return false;
     }
 
-    if(ssh_handle_key_exchange(m_session)) {
+    if (ssh_handle_key_exchange(m_session)) {
 		PTRACE(1, "ssh_handle_key_exchange failed: " << ssh_get_error(m_session));
 		SNMP_TRAP(7, SNMPError, Network, "SSH key exchange failed");
 		return false;
@@ -429,6 +437,13 @@ PBoolean SSHStatusClient::Accept(PSocket & socket)
 	// set handle for SocketsReader
 	os_handle = ssh_get_fd(m_session);
 
+#ifdef LARGE_FDSET
+	socklen_t addr_len = sizeof(peeraddr);
+	if (getpeername(os_handle, ((struct sockaddr *)&peeraddr), &addr_len) == 0) {
+		PTRACE(1, "SSH\tgetpeername() failed");
+	}
+#endif
+
 	Address raddr, laddr;
 	WORD rport = 0, lport = 0;
 	GetPeerAddress(raddr, rport);
@@ -524,6 +539,16 @@ bool SSHStatusClient::Authenticate()
 			                break;
 						case SSH_CHANNEL_REQUEST_PTY:
 		                    ssh_message_channel_request_reply_success(m_message);
+			                break;
+						case SSH_CHANNEL_REQUEST_EXEC:
+							{
+								shell = true;
+								const char * cmd = ssh_message_channel_request_command(m_message);
+								OnCommand(cmd);
+								Flush();
+			                    ssh_message_channel_request_reply_success(m_message);
+								m_done = true;
+							}
 			                break;
 						case SSH_CHANNEL_REQUEST_SHELL:
 							shell = true;
@@ -533,7 +558,7 @@ bool SSHStatusClient::Authenticate()
 			                ssh_message_reply_default(m_message);	// deny X11 forwarding
 			                break;
 						default:
-							PTRACE(3, "Unhandled SSH_REQUEST_CHANNEL message " << ssh_message_type(m_message) << " subtype " << ssh_message_subtype(m_message));
+							PTRACE(3, "Unhandled SSH_REQUEST_CHANNEL message subtype " << ssh_message_subtype(m_message));
 							break;
 					}
 					break;
@@ -568,7 +593,7 @@ bool SSHStatusClient::AuthenticateUser()
 	} else
 		PTRACE(5, "STATUS\tPassword mismatch for user " << login);
 
-	PProcess::Sleep(delay * 1000);
+	PThread::Sleep(delay * 1000);
 
 	return false;
 }
@@ -681,7 +706,8 @@ int TelnetSocket::ReadChar()
 
 	while (!hasData) {
 		if (!TCPSocket::Read(&currentByte, 1)) {
-			if (GetErrorCode(PSocket::LastReadError) != PSocket::Timeout) {
+			const PSocket::Errors err = GetErrorCode(PSocket::LastReadError);
+			if ((err != PSocket::Timeout) && (err != PSocket::NoError)) {
 				PTRACE(3, "TELNET\t" << GetName() << " closed the connection ("
 					<< GetErrorCode(PSocket::LastReadError) << '/' 
 					<< GetErrorNumber(PSocket::LastReadError) << ": "
@@ -846,17 +872,31 @@ GkStatus::GkStatus() : Singleton<GkStatus>("GkStatus"), SocketsReader(500)
 	Execute();
 }
 
+GkStatus::~GkStatus()
+{
+	ReadLock lock(m_listmutex);
+	for (iterator i = m_sockets.begin(); i != m_sockets.end(); ++i) {
+		(*i)->Close();
+	}
+}
+
 void GkStatus::AuthenticateClient(StatusClient * newClient)
 {
 	if ((m_statusClients++ < m_maxStatusClients) && (newClient->Authenticate())) {
 		newClient->SetTraceLevel(GkConfig()->GetInteger("StatusTraceLevel", MAX_STATUS_TRACE_LEVEL));
 		PTRACE(1, "STATUS\tNew client authenticated successfully: " << newClient->WhoAmI()
-			<< ", login: " << newClient->GetUser()
-			);
+			<< ", login: " << newClient->GetUser());
 		// the welcome messages
 		newClient->WriteString(PrintGkVersion());
 		newClient->Flush();
 		AddSocket(newClient);
+		if (newClient->IsDone()) {
+			while (newClient->IsBusy()) {
+				PThread::Sleep(100);	// give forked threads time to finish
+			}
+			newClient->Flush();
+			newClient->Close();
+		}
 	} else {
 		PTRACE(3, "STATUS\tNew client rejected: " << newClient->WhoAmI() << ", login: " << newClient->GetUser());
 		newClient->WriteString("\r\nAccess forbidden!\r\n");
@@ -911,7 +951,7 @@ bool GkStatus::DisconnectSession(
 	/// session ID (instance number) for the status client to be disconnected
 	int instanceNo,
 	/// status interface client that requested disconnect
-	StatusClient* requestingClient
+	StatusClient * requestingClient
 	)
 {
 	ReadLock lock(m_listmutex);
@@ -949,13 +989,13 @@ public:
 
 private:
 	/// status interface client to send the information to
-	StatusClient* m_requestingClient;
+	StatusClient * m_requestingClient;
 };
 
 
 void GkStatus::ShowUsers(
 	/// client that requested the list of all active clients
-	StatusClient* requestingClient
+	StatusClient * requestingClient
 	) const
 {
 	ReadLock lock(m_listmutex);
@@ -1092,12 +1132,10 @@ void GkStatus::OnStart()
 	m_commands["printallconfigswitches"] = e_PrintAllConfigSwitches;
 }
 
-void GkStatus::ReadSocket(
-	IPSocket* clientSocket
-	)
+void GkStatus::ReadSocket(IPSocket * clientSocket)
 {
 	PString cmd;
-	StatusClient* client = static_cast<StatusClient*>(clientSocket);
+	StatusClient * client = static_cast<StatusClient*>(clientSocket);
 	if (client->ReadCommand(cmd) && !cmd.IsEmpty()) {
 		client->OnCommand(cmd);
 	}
@@ -1109,7 +1147,7 @@ void GkStatus::CleanUp()
 		PWaitAndSignal lock(m_rmutex);
 		iterator iter = m_removed.begin();
 		while (iter != m_removed.end()) {
-			StatusClient *client = static_cast<StatusClient *>(*iter);
+			StatusClient * client = static_cast<StatusClient *>(*iter);
 			if (client && !client->IsBusy()) {
 				iter = m_removed.erase(iter);
 				--m_rmsize;
@@ -1131,6 +1169,7 @@ StatusClient::StatusClient(
 	m_numExecutingCommands(0),
 	m_instanceNo(instanceNo),
 	m_traceLevel(MAX_STATUS_TRACE_LEVEL),
+	m_done(false),
 	m_isFilteringActive(false),
 	m_handlePasswordRule(true)
 {
