@@ -3,7 +3,7 @@
 // Neighboring System for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2002-2003
-// Copyright (c) 2004-2012, Jan Willamowius
+// Copyright (c) 2004-2013, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -46,6 +46,7 @@ namespace Neighbors {
 
 const char *NeighborSection = "RasSrv::Neighbors";
 const char *LRQFeaturesSection = "RasSrv::LRQFeatures";
+const char *RoutedSec = "RoutedMode";
 
 static const char OID_MD5[] = "1.2.840.113549.2.5";
 
@@ -367,7 +368,7 @@ bool Neighbor::SetProfile(const PString & id, const PString & type)
 	m_H46018Server = Toolkit::AsBool(config->GetString(section, "H46018Server", "0"));
 #endif
 #ifdef HAS_TLS
-	m_useTLS = Toolkit::AsBool(config->GetString(section, "UseTLS", "0"));
+	m_useTLS = Toolkit::AsBool(config->GetString(section, "UseTLS", "0")); // forced use of TLS
 #endif
 
 	SetForwardedInfo(section);
@@ -721,7 +722,7 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const AdmissionRequest & re
 	lrq.m_gatekeeperIdentifier = Toolkit::GKName();
 	lrq.IncludeOptionalField(H225_LocationRequest::e_nonStandardData);
 	lrq.m_nonStandardData.m_nonStandardIdentifier.SetTag(H225_NonStandardIdentifier::e_h221NonStandard);
-	H225_H221NonStandard &t35 = lrq.m_nonStandardData.m_nonStandardIdentifier;
+	H225_H221NonStandard & t35 = lrq.m_nonStandardData.m_nonStandardIdentifier;
 	t35.m_t35CountryCode = Toolkit::t35cPoland;
 	t35.m_manufacturerCode = Toolkit::t35mGnuGk;
 	t35.m_t35Extension = Toolkit::t35eNeighborId;
@@ -741,24 +742,69 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const AdmissionRequest & re
 		lrq.m_callIdentifier = arq.m_callIdentifier;
 	}
 
+#if defined(HAS_TLS) && defined(HAS_H460)
+	// H.460.22
+	if (RasServer::Instance()->IsGKRouted()) {
+		// in routed mode we signal gatekeeper capabilities
+		if (Toolkit::Instance()->IsTLSEnabled()) {
+			// include H.460.22 in supported features
+			H460_FeatureStd h46022 = H460_FeatureStd(22);
+			H460_FeatureStd settings;
+			settings.Add(Std22_Priority, H460_FeatureContent(1, 8)); // Priority=1, type=number8
+			WORD tlsSignalPort = (WORD)GkConfig()->GetInteger(RoutedSec, "TLSCallSignalPort", GK_DEF_TLS_CALL_SIGNAL_PORT);
+			H225_TransportAddress h225Addr = m_rasSrv->GetRasAddress(GetIP());
+			SetH225Port(h225Addr, tlsSignalPort);
+			H323TransportAddress signalAddr = h225Addr;
+			settings.Add(Std22_ConnectionAddress, H460_FeatureContent(signalAddr));
+			h46022.Add(Std22_TLS, H460_FeatureContent(settings.GetCurrentTable()));
+			lrq.IncludeOptionalField(H225_LocationRequest::e_featureSet);
+			lrq.m_featureSet.IncludeOptionalField(H225_FeatureSet::e_supportedFeatures);
+			H225_ArrayOf_FeatureDescriptor & desc = lrq.m_featureSet.m_supportedFeatures;
+			PINDEX lPos = desc.GetSize();
+			desc.SetSize(lPos + 1);
+			desc[lPos] = h46022;
+		}
+	} else {
+		// in direct mode we signal / forward endpoint capabilities
+		if (arq.HasOptionalField(H225_AdmissionRequest::e_featureSet)) {
+			H460_FeatureSet fs = H460_FeatureSet(arq.m_featureSet);
+			if (fs.HasFeature(22)) {
+				// copy feature 22 from endpoint ARQ to LRQ
+				H460_FeatureStd * h46022 = (H460_FeatureStd *)fs.GetFeature(22);
+				lrq.IncludeOptionalField(H225_LocationRequest::e_featureSet);
+				lrq.m_featureSet.IncludeOptionalField(H225_FeatureSet::e_supportedFeatures);
+				H225_ArrayOf_FeatureDescriptor & desc = lrq.m_featureSet.m_supportedFeatures;
+				PINDEX lPos = desc.GetSize();
+				desc.SetSize(lPos + 1);
+				desc[lPos] = *h46022;
+			}
+		}
+	}
+#endif
+
 #ifdef HAS_H46023
-    /// STD24  NAT Support
-    if (Toolkit::Instance()->IsH46023Enabled()) {
-        H460_FeatureStd std24 = H460_FeatureStd(24);
-        int sz = lrq.m_genericData.GetSize();
-        lrq.m_genericData.SetSize(sz+1);
-        lrq.m_genericData[sz] = std24;
-    }
+	/// STD24  NAT Support
+	if (Toolkit::Instance()->IsH46023Enabled()) {
+		H460_FeatureStd std24 = H460_FeatureStd(24);
+		int sz = lrq.m_genericData.GetSize();
+		lrq.m_genericData.SetSize(sz + 1);
+		lrq.m_genericData[sz] = std24;
+	}
 #endif
 #ifdef HAS_H460VEN
-     /// OID9  'Remote endpoint vendor info THIS IS "1.3.6.1.4.1.17090.0.9" NOT H.460.9	- SH
-     H460_FeatureOID foid9 = H460_FeatureOID(OID9);	 
-     int sz = lrq.m_genericData.GetSize();	 
-     lrq.m_genericData.SetSize(sz+1);	 
-     lrq.m_genericData[sz] = foid9;
+	/// OID9  'Remote endpoint vendor info THIS IS "1.3.6.1.4.1.17090.0.9" NOT H.460.9	- SH
+	H460_FeatureOID foid9 = H460_FeatureOID(OID9);	 
+	int sz = lrq.m_genericData.GetSize();	 
+	lrq.m_genericData.SetSize(sz + 1);	 
+	lrq.m_genericData[sz] = foid9;
 #endif
-     if (lrq.m_genericData.GetSize() > 0)
-          lrq.IncludeOptionalField(H225_LocationRequest::e_genericData);
+#ifdef HAS_LANGUAGE
+	if (GkConfig()->GetBoolean("RasSrv::LRQFeatures", "EnableLanguageRouting", false)) {
+		lrq.IncludeOptionalField(H225_LocationRequest::e_language); 
+	}
+#endif
+	if (lrq.m_genericData.GetSize() > 0)
+		lrq.IncludeOptionalField(H225_LocationRequest::e_genericData);
 
 	return true;
 }
@@ -769,7 +815,7 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const SetupRequest & reques
 	lrq.m_gatekeeperIdentifier = Toolkit::GKName();
 	lrq.IncludeOptionalField(H225_LocationRequest::e_nonStandardData);
 	lrq.m_nonStandardData.m_nonStandardIdentifier.SetTag(H225_NonStandardIdentifier::e_h221NonStandard);
-	H225_H221NonStandard &t35 = lrq.m_nonStandardData.m_nonStandardIdentifier;
+	H225_H221NonStandard & t35 = lrq.m_nonStandardData.m_nonStandardIdentifier;
 	t35.m_t35CountryCode = Toolkit::t35cPoland;
 	t35.m_manufacturerCode = Toolkit::t35mGnuGk;
 	t35.m_t35Extension = Toolkit::t35eNeighborId;
@@ -790,6 +836,53 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const SetupRequest & reques
 		lrq.m_callIdentifier = setup.m_callIdentifier;
 	}
 
+#if defined(HAS_TLS) && defined(HAS_H460)
+	// H.460.22
+	// if we generate LRQs from Setup messages we must be in routed mode, so we signal gatekeeper capabilities
+	if (Toolkit::Instance()->IsTLSEnabled()) {
+		// include H.460.22 in supported features
+		H460_FeatureStd h46022 = H460_FeatureStd(22);
+		H460_FeatureStd settings;
+		settings.Add(Std22_Priority, H460_FeatureContent(1, 8)); // Priority=1, type=number8
+		WORD tlsSignalPort = (WORD)GkConfig()->GetInteger(RoutedSec, "TLSCallSignalPort", GK_DEF_TLS_CALL_SIGNAL_PORT);
+		H225_TransportAddress h225Addr = m_rasSrv->GetRasAddress(GetIP());
+		SetH225Port(h225Addr, tlsSignalPort);
+		H323TransportAddress signalAddr = h225Addr;
+		settings.Add(Std22_ConnectionAddress, H460_FeatureContent(signalAddr));
+		h46022.Add(Std22_TLS, H460_FeatureContent(settings.GetCurrentTable()));
+		lrq.IncludeOptionalField(H225_LocationRequest::e_featureSet);
+		lrq.m_featureSet.IncludeOptionalField(H225_FeatureSet::e_supportedFeatures);
+		H225_ArrayOf_FeatureDescriptor & desc = lrq.m_featureSet.m_supportedFeatures;
+		PINDEX lPos = desc.GetSize();
+		desc.SetSize(lPos + 1);
+		desc[lPos] = h46022;
+	}
+#endif
+
+#ifdef HAS_H46023
+	/// STD24  NAT Support
+	if (Toolkit::Instance()->IsH46023Enabled()) {
+		H460_FeatureStd std24 = H460_FeatureStd(24);
+		int sz = lrq.m_genericData.GetSize();
+		lrq.m_genericData.SetSize(sz + 1);
+		lrq.m_genericData[sz] = std24;
+	}
+#endif
+#ifdef HAS_H460VEN
+	/// OID9  'Remote endpoint vendor info THIS IS "1.3.6.1.4.1.17090.0.9" NOT H.460.9	- SH
+	H460_FeatureOID foid9 = H460_FeatureOID(OID9);	 
+	int sz = lrq.m_genericData.GetSize();	 
+	lrq.m_genericData.SetSize(sz + 1);	 
+	lrq.m_genericData[sz] = foid9;
+#endif
+#ifdef HAS_LANGUAGE
+	if (GkConfig()->GetBoolean("RasSrv::LRQFeatures", "EnableLanguageRouting", false)) {
+		lrq.IncludeOptionalField(H225_LocationRequest::e_language); 
+	}
+#endif
+	if (lrq.m_genericData.GetSize() > 0)
+		lrq.IncludeOptionalField(H225_LocationRequest::e_genericData);
+
 	return true;
 }
 
@@ -799,7 +892,7 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const FacilityRequest & /*r
 	lrq.m_gatekeeperIdentifier = Toolkit::GKName();
 	lrq.IncludeOptionalField(H225_LocationRequest::e_nonStandardData);
 	lrq.m_nonStandardData.m_nonStandardIdentifier.SetTag(H225_NonStandardIdentifier::e_h221NonStandard);
-	H225_H221NonStandard &t35 = lrq.m_nonStandardData.m_nonStandardIdentifier;
+	H225_H221NonStandard & t35 = lrq.m_nonStandardData.m_nonStandardIdentifier;
 	t35.m_t35CountryCode = Toolkit::t35cPoland;
 	t35.m_manufacturerCode = Toolkit::t35mGnuGk;
 	t35.m_t35Extension = Toolkit::t35eNeighborId;
@@ -808,6 +901,53 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const FacilityRequest & /*r
 	lrq.IncludeOptionalField(H225_LocationRequest::e_canMapAlias);
 	lrq.m_canMapAlias = TRUE;
 
+#if defined(HAS_TLS) && defined(HAS_H460)
+	// H.460.22
+	// if we generate LRQs from Facility messages we must be in routed mode, so we signal gatekeeper capabilities
+	if (Toolkit::Instance()->IsTLSEnabled()) {
+		// include H.460.22 in supported features
+		H460_FeatureStd h46022 = H460_FeatureStd(22);
+		H460_FeatureStd settings;
+		settings.Add(Std22_Priority, H460_FeatureContent(1, 8)); // Priority=1, type=number8
+		WORD tlsSignalPort = (WORD)GkConfig()->GetInteger(RoutedSec, "TLSCallSignalPort", GK_DEF_TLS_CALL_SIGNAL_PORT);
+		H225_TransportAddress h225Addr = m_rasSrv->GetRasAddress(GetIP());
+		SetH225Port(h225Addr, tlsSignalPort);
+		H323TransportAddress signalAddr = h225Addr;
+		settings.Add(Std22_ConnectionAddress, H460_FeatureContent(signalAddr));
+		h46022.Add(Std22_TLS, H460_FeatureContent(settings.GetCurrentTable()));
+		lrq.IncludeOptionalField(H225_LocationRequest::e_featureSet);
+		lrq.m_featureSet.IncludeOptionalField(H225_FeatureSet::e_supportedFeatures);
+		H225_ArrayOf_FeatureDescriptor & desc = lrq.m_featureSet.m_supportedFeatures;
+		PINDEX lPos = desc.GetSize();
+		desc.SetSize(lPos + 1);
+		desc[lPos] = h46022;
+	}
+#endif
+
+#ifdef HAS_H46023
+	/// STD24  NAT Support
+	if (Toolkit::Instance()->IsH46023Enabled()) {
+		H460_FeatureStd std24 = H460_FeatureStd(24);
+		int sz = lrq.m_genericData.GetSize();
+		lrq.m_genericData.SetSize(sz + 1);
+		lrq.m_genericData[sz] = std24;
+	}
+#endif
+#ifdef HAS_H460VEN
+	/// OID9  'Remote endpoint vendor info THIS IS "1.3.6.1.4.1.17090.0.9" NOT H.460.9	- SH
+	H460_FeatureOID foid9 = H460_FeatureOID(OID9);	 
+	int sz = lrq.m_genericData.GetSize();	 
+	lrq.m_genericData.SetSize(sz + 1);	 
+	lrq.m_genericData[sz] = foid9;
+#endif
+#ifdef HAS_LANGUAGE
+	if (GkConfig()->GetBoolean("RasSrv::LRQFeatures", "EnableLanguageRouting", false)) {
+		lrq.IncludeOptionalField(H225_LocationRequest::e_language); 
+	}
+#endif
+	if (lrq.m_genericData.GetSize() > 0)
+		lrq.IncludeOptionalField(H225_LocationRequest::e_genericData);
+
 	return true;
 }
 
@@ -1031,7 +1171,11 @@ public:
 	bool IsTraversalServer() const { return m_h46018_server; }
 	bool IsTraversalZone() const { return m_h46018_client || m_h46018_server; }
 	bool IsH46024Supported() const;
+	bool IsTLSNegotiated();
 	bool UseTLS() const { return m_useTLS; }
+	bool HasVendorInfo() const;
+	bool SupportLanguages() const;
+	PStringList GetLanguages() const;
 
 	// override from class RasRequester
 	virtual bool IsExpected(const RasMsg *) const;
@@ -1126,15 +1270,85 @@ bool LRQRequester::IsH46024Supported() const
 	if (!m_result)
 		return false;
 
-	const H225_LocationConfirm *lcf = &(H225_LocationConfirm &)(*m_result)->m_recvRAS;
-	if (lcf && lcf->HasOptionalField(H225_LocationConfirm::e_genericData)) {
-		H460_FeatureSet fs = H460_FeatureSet(lcf->m_genericData);
+	const H225_LocationConfirm & lcf = (*m_result)->m_recvRAS;
+	if (lcf.HasOptionalField(H225_LocationConfirm::e_genericData)) {
+		H460_FeatureSet fs = H460_FeatureSet(lcf.m_genericData);
 		if (fs.HasFeature(24))
 			return true;
 	}
 	return false;
 }
 
+bool LRQRequester::IsTLSNegotiated()
+{
+	if (!m_result)
+		return false;
+
+	H225_LocationConfirm & lcf = (*m_result)->m_recvRAS;
+	if (lcf.HasOptionalField(H225_LocationConfirm::e_featureSet)) {
+		H460_FeatureSet fs = H460_FeatureSet(lcf.m_featureSet);
+		if (fs.HasFeature(22)) {
+			H460_FeatureStd * secfeat = (H460_FeatureStd *)fs.GetFeature(22);
+			if (secfeat->Contains(Std22_TLS)) {
+				m_useTLS = true;
+				H460_FeatureParameter & tlsparam = secfeat->Value(Std22_TLS);
+				H460_FeatureStd settings;
+				settings.SetCurrentTable(tlsparam);
+				if (settings.Contains(Std22_ConnectionAddress)) {
+					H323TransportAddress tlsAddr = settings.Value(Std22_ConnectionAddress);
+					// put TLS address into callSignalAddress, so we don't have to check both when creating the route
+					lcf.m_callSignalAddress = H323ToH225TransportAddress(tlsAddr);
+				}
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
+bool LRQRequester::HasVendorInfo() const
+{
+	if (!m_result)
+		return false;
+#ifdef HAS_H460VEN
+	const H225_LocationConfirm & lcf = (*m_result)->m_recvRAS;
+	if (lcf.HasOptionalField(H225_LocationConfirm::e_genericData)) {
+		H460_FeatureSet fs = H460_FeatureSet(lcf.m_genericData);
+		if (fs.HasFeature(OpalOID(OID9)))
+			return true;
+	}
+#endif
+	return false;
+}
+
+bool LRQRequester::SupportLanguages() const
+{
+	if (!m_result)
+		return false;
+#ifdef HAS_LANGUAGE
+	return GkConfig()->GetBoolean(LRQFeaturesSection, "EnableLanguageRouting", false);
+#else
+    return false;
+#endif
+}
+
+PStringList LRQRequester::GetLanguages() const
+{
+	PStringList languages;
+
+	if (!m_result)
+		return languages;
+
+#ifdef HAS_LANGUAGE
+	const H225_LocationConfirm & lcf = (*m_result)->m_recvRAS;
+	if (lcf.HasOptionalField(H225_LocationConfirm::e_language)) {
+		H323GetLanguages(languages, lcf.m_language);
+		return languages;
+	}
+#endif
+	return languages;
+}
+
 bool LRQRequester::IsExpected(const RasMsg * ras) const
 {
 	return RasHandler::IsExpected(ras) && (ras->GetSeqNum() == m_seqNum);
@@ -1178,6 +1392,7 @@ void LRQRequester::Process(RasMsg * ras)
 				m_neighbor_used = req.m_neighbor->GetId(); // record neighbor used
 #ifdef HAS_TLS
 				m_useTLS = req.m_neighbor->UseTLS();
+				// TODO22: also set m_useTLS by H.460.22 ? but it could be in direct mode and the capabilities are just for the one endpoint...
 #endif
 				m_h46018_client = req.m_neighbor->IsH46018Client();
 				m_h46018_server = req.m_neighbor->IsH46018Server();
@@ -1512,16 +1727,25 @@ bool NeighborPolicy::OnRequest(AdmissionRequest & arq_obj)
 	if (request.Send(m_neighbors)) {
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
-#if defined(HAS_H460) || defined (HAS_TLS)
-			if ((request.UseTLS() || request.IsTraversalZone() || request.IsH46024Supported()) && RasServer::Instance()->IsGKRouted()) {
-				// overwrite callSignalAddress and replace with ours, we must proxy this call
-				endptr ep = RegistrationTable::Instance()->FindByEndpointId(arq_obj.GetRequest().m_endpointIdentifier); // should not be null
-				if (ep) {
-					 PIPSocket::Address epip;
-					if (GetIPFromTransportAddr(ep->GetCallSignalAddress(), epip))
-						route.m_destAddr = RasServer::Instance()->GetCallSignalAddress(epip);
+#if defined(HAS_H460) || defined (HAS_TLS) || defined (HAS_LANGUAGE)
+			// create an EPRec to remember the traversal settings etc.				
+			if (request.UseTLS() || request.IsTLSNegotiated() || request.IsTraversalZone() || request.IsH46024Supported()
+				|| request.HasVendorInfo() || request.SupportLanguages()) {
+				// overwrite callSignalAddress and replace with ours, we must proxy this call (works only in routed mode)
+				if (request.IsTraversalZone() && RasServer::Instance()->IsGKRouted()) {
+					endptr ep = RegistrationTable::Instance()->FindByEndpointId(arq_obj.GetRequest().m_endpointIdentifier); // should not be null
+					if (ep) {
+						PIPSocket::Address epip;
+						if (GetIPFromTransportAddr(ep->GetCallSignalAddress(), epip))
+							route.m_destAddr = RasServer::Instance()->GetCallSignalAddress(epip);
+
+						if (request.SupportLanguages() && !route.SetLanguages(ep->GetLanguages(),request.GetLanguages())) {
+							PTRACE(4, m_name << "\tPolicy found but rejected as no common language");
+							return false;
+						}
+					}
 				}
-				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
+				// create the EPRec
 				H225_RasMessage ras;
 				ras.SetTag(H225_RasMessage::e_locationConfirm);
 				H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
@@ -1536,7 +1760,7 @@ bool NeighborPolicy::OnRequest(AdmissionRequest & arq_obj)
 					route.m_destEndpoint->SetTraversalRole(TraversalServer);
 				}
 				if (request.IsTraversalServer()) {
-                    route.m_destEndpoint->SetTraversalRole(TraversalClient);
+					route.m_destEndpoint->SetTraversalRole(TraversalClient);
 				}
 			}
 #endif
@@ -1614,7 +1838,7 @@ bool NeighborPolicy::OnRequest(LocationRequest & lrq_obj)
 					PString data = param->m_data.AsString() + ":" + PString(request.GetReqNumber());
 					rip.IncludeOptionalField(H225_RequestInProgress::e_nonStandardData);
 					rip.m_nonStandardData.m_nonStandardIdentifier.SetTag(H225_NonStandardIdentifier::e_h221NonStandard);
-					H225_H221NonStandard &t35 = rip.m_nonStandardData.m_nonStandardIdentifier;
+					H225_H221NonStandard & t35 = rip.m_nonStandardData.m_nonStandardIdentifier;
 					t35.m_t35CountryCode = Toolkit::t35cPoland;
 					t35.m_manufacturerCode = Toolkit::t35mGnuGk;
 					t35.m_t35Extension = Toolkit::t35eNeighborId;
@@ -1680,7 +1904,8 @@ bool NeighborPolicy::OnRequest(SetupRequest & setup_obj)
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
 #if defined(HAS_H460) || defined (HAS_TLS)
-			if (request.UseTLS() || request.IsTraversalZone() || request.IsH46024Supported()) {
+			if ((request.UseTLS() || request.IsTLSNegotiated() || request.IsTraversalZone() || request.IsH46024Supported())
+				|| request.HasVendorInfo() || request.SupportLanguages()) {
 				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
 				H225_RasMessage ras;
 				ras.SetTag(H225_RasMessage::e_locationConfirm);
@@ -1725,7 +1950,7 @@ bool NeighborPolicy::OnRequest(FacilityRequest & facility_obj)
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
 #if defined(HAS_H460) || defined (HAS_TLS)
-			if (request.UseTLS() || request.IsTraversalZone() || request.IsH46024Supported()) {
+			if (request.UseTLS() || request.IsTLSNegotiated() || request.IsTraversalZone() || request.IsH46024Supported()) {
 				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
 				H225_RasMessage ras;
 				ras.SetTag(H225_RasMessage::e_locationConfirm);
@@ -1885,8 +2110,27 @@ Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress
 					if (pRequest && pRequest->Send(nb)) {
 						if (H225_LocationConfirm * lcf = pRequest->WaitForDestination(m_neighborTimeout)) {
 							Route * route = new Route(m_name, lcf->m_callSignalAddress);
+#ifdef HAS_LANGUAGE
+							if (pRequest->SupportLanguages()) {
+								endptr ep;
+								if (dynamic_cast<AdmissionRequest *>(&request))
+									ep = RegistrationTable::Instance()->FindByEndpointId((((AdmissionRequest &)request).GetRequest().m_endpointIdentifier));
+								else if (dynamic_cast<SetupRequest *>(&request))
+									ep = RegistrationTable::Instance()->FindByEndpointId((((SetupRequest &)request).GetRequest().m_endpointIdentifier));
+								else if (dynamic_cast<LocationRequest *>(&request))
+									ep = RegistrationTable::Instance()->FindByEndpointId((((LocationRequest &)request).GetRequest().m_endpointIdentifier));
+
+								if (!ep || !route->SetLanguages(ep->GetLanguages(), pRequest->GetLanguages())) {
+									PTRACE(4, m_name << "\tPolicy found but rejected as no common language");
+									delete pRequest;
+									delete nb;
+									delete route;
+									return NULL;
+								}
+							}
+#endif
 #ifdef HAS_H460
-							if (pRequest->IsH46024Supported()) {
+							if (pRequest->IsH46024Supported() || pRequest->HasVendorInfo()) {
 								H225_RasMessage ras;
 								ras.SetTag(H225_RasMessage::e_locationConfirm);
 								H225_LocationConfirm & conf = (H225_LocationConfirm &)ras;
@@ -2005,6 +2249,12 @@ Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly,
 // used for ARQs and Setups
 bool SRVPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
+	PString service = request.GetServiceType();
+	if (!service && (service != m_instance)) {
+		PTRACE(4, "ROUTING\tPolicy " << m_name << " not supported for service " << service);
+		return false;
+	}
+
 	bool localonly = dynamic_cast<LocationRequest *>(&request) && !m_resolveNonLocalLRQs;
 
 	Route * route = NULL;
@@ -2142,8 +2392,26 @@ bool RDSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 				if (pRequest && pRequest->Send(nb)) {
 					if (H225_LocationConfirm *lcf = pRequest->WaitForDestination(m_neighborTimeout)) {
 						Route route(m_name, lcf->m_callSignalAddress);
+#ifdef HAS_LANGUAGE
+						if (pRequest->SupportLanguages()) {
+							endptr ep;
+							if (dynamic_cast<AdmissionRequest *>(&request))
+								ep = RegistrationTable::Instance()->FindByEndpointId((((AdmissionRequest &)request).GetRequest().m_endpointIdentifier));
+							else if (dynamic_cast<SetupRequest *>(&request))
+								ep = RegistrationTable::Instance()->FindByEndpointId((((SetupRequest &)request).GetRequest().m_endpointIdentifier));
+							else if (dynamic_cast<LocationRequest *>(&request))
+								ep = RegistrationTable::Instance()->FindByEndpointId((((LocationRequest &)request).GetRequest().m_endpointIdentifier));
+
+							if (!ep || !route.SetLanguages(ep->GetLanguages(), pRequest->GetLanguages())) {
+								PTRACE(4, m_name << "\tPolicy found but rejected as no common language");
+								delete pRequest;
+								delete nb;
+								return false;
+							}
+						}
+#endif
 #ifdef HAS_H460
-						if (pRequest->IsH46024Supported()) {
+						if (pRequest->IsH46024Supported() || pRequest->HasVendorInfo()) {
 							H225_RasMessage ras;
 							ras.SetTag(H225_RasMessage::e_locationConfirm);
 							H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
@@ -2158,7 +2426,7 @@ bool RDSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 						return true;
 					}
 				}
-				PTRACE(4, "ROUTING\tDNS RDS LRQ Error for " << domain << " at " << ipaddr);
+				PTRACE(4, m_name << "\tDNS RDS LRQ Error for " << domain << " at " << ipaddr);
 				delete pRequest;
 				delete nb;
 			}
@@ -2258,8 +2526,26 @@ bool NeighborSqlPolicy::ResolveRoute(RoutingRequest & request, DestinationRoutes
 	if (pRequest && pRequest->Send(nb)) {
 		if (H225_LocationConfirm * lcf = pRequest->WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
+#ifdef HAS_LANGUAGE
+			if (pRequest->SupportLanguages()) {
+				endptr ep;
+				if (dynamic_cast<AdmissionRequest *>(&request))
+					ep = RegistrationTable::Instance()->FindByEndpointId((((AdmissionRequest &)request).GetRequest().m_endpointIdentifier));
+				else if (dynamic_cast<SetupRequest *>(&request))
+					ep = RegistrationTable::Instance()->FindByEndpointId((((SetupRequest &)request).GetRequest().m_endpointIdentifier));
+				else if (dynamic_cast<LocationRequest *>(&request))
+					ep = RegistrationTable::Instance()->FindByEndpointId((((LocationRequest &)request).GetRequest().m_endpointIdentifier));
+
+				if (!ep || !route.SetLanguages(ep->GetLanguages(), pRequest->GetLanguages())) {
+					PTRACE(4, m_name << "\tPolicy found but rejected as no common language");
+					delete pRequest;
+					delete nb;
+					return false;
+				}
+			}
+#endif
 #ifdef HAS_H460
-			if (pRequest->IsH46024Supported()) {
+			if (pRequest->IsH46024Supported() || pRequest->HasVendorInfo()) {
 				H225_RasMessage ras;
 				ras.SetTag(H225_RasMessage::e_locationConfirm);
 				H225_LocationConfirm & conf = (H225_LocationConfirm &)ras;
