@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef PROXYCHANNEL_H
-#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.155.2.1 2013/09/19 16:48:39 willamowius Exp $"
+#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.163 2014/01/16 10:14:32 willamowius Exp $"
 
 #include <vector>
 #include <list>
@@ -126,7 +126,7 @@ private:
 
 class TCPProxySocket : public ServerSocket, public ProxySocket {
 public:
-	TCPProxySocket(const char *, TCPProxySocket * = NULL, WORD = 0);
+	TCPProxySocket(const char * t, TCPProxySocket * s = NULL, WORD p = 0);
 	virtual ~TCPProxySocket();
 
 #ifndef LARGE_FDSET
@@ -213,6 +213,7 @@ public:
 	void SetUsesH46019() { m_useH46019 = true; }
 	bool UsesH46019() const { return m_useH46019; }
 	void SetH46019UniDirectional(bool val) { m_h46019uni = val; }
+	void AddKeepAlivePT(BYTE pt);
 	void SetMultiplexDestination(const H323TransportAddress & toAddress, H46019Side side);
 	void SetMultiplexID(PUInt32b multiplexID, H46019Side side);
 	void SetMultiplexSocket(int multiplexSocket, H46019Side side);
@@ -258,6 +259,10 @@ protected:
 	bool m_h46019uni;
 	bool m_h46019DetectionDone;
 	PMutex m_h46019DetectionLock;
+	bool m_checkH46019KeepAlivePT;
+	WORD m_keepAlivePT_1;
+	WORD m_keepAlivePT_2;
+	PTime m_channelStartTime;
 	// two (!), one or zero parties in a call through a UDPProxySocket may by multiplexed
 	// UDPProxySocket always receives regular RTP, but may send out multiplexed
 	H323TransportAddress m_multiplexDestination_A;	// OLC side of first logical channel in this session
@@ -304,6 +309,7 @@ public:
 	bool IsH245Tunneling() const { return m_h245Tunneling; }
 	bool IsH245TunnelingTranslation() const { return m_h245TunnelingTranslation; }
 	PASN_OctetString * GetNextQueuedH245Message();
+	unsigned GetH245MessageQueueSize() const { return  m_h245Queue.size(); }
 	bool HandleH245Mesg(PPER_Stream &, bool & suppress, H245Socket * h245sock = NULL);
 	void SendPostDialDigits();
 	void OnH245ChannelClosed() { m_h245socket = NULL; }
@@ -363,7 +369,7 @@ public:
 	bool IsTraversalClient() const;
 	bool IsTraversalServer() const;
 	bool CreateRemote(const H225_TransportAddress & addr);
-	bool OnSCICall(const H225_CallIdentifier & callID, H225_TransportAddress sigAdr);
+	bool OnSCICall(const H225_CallIdentifier & callID, H225_TransportAddress sigAdr, bool useTLS);
 	bool IsCallFromTraversalServer() const { return m_callFromTraversalServer; }
 	bool IsCallToTraversalServer() const { return m_callToTraversalServer; }
 	void SetSessionMultiplexDestination(WORD session, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
@@ -390,6 +396,7 @@ protected:
 	void OnFacility(SignalingMsg *msg);
 	void OnProgress(SignalingMsg *msg);
 	void OnInformation(SignalingMsg *msg);
+	void OnStatus(SignalingMsg *msg);
 
 	bool OnTunneledH245(H225_ArrayOf_PASN_OctetString &, bool & suppress);
 	bool OnFastStart(H225_ArrayOf_PASN_OctetString &, bool);
@@ -551,11 +558,12 @@ public:
 	virtual bool Read(void * buf, int sz);
 	virtual int GetLastReadCount() const { return m_lastReadCount; }
 	virtual bool Write(const void * buf, int sz);
+	virtual int GetLastWriteCount() const { return m_lastWriteCount; }
 	virtual void Dispatch();
 
 protected:
 	SSL * m_ssl;
-	int m_lastReadCount;
+	int m_lastReadCount, m_lastWriteCount;
 };
 
 #endif // HAS_TLS
@@ -782,6 +790,7 @@ private:
 #ifdef HAS_H46017
 	bool m_h46017Enabled;
 #endif
+	bool m_proxyHandlerHighPrio;
 };
 
 class HandlerList {
