@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASTBL_H
-#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.277.2.1 2013/09/26 22:37:13 willamowius Exp $"
+#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.287 2014/03/14 10:52:56 willamowius Exp $"
 
 #include <list>
 #include <map>
@@ -323,6 +323,20 @@ public:
 
 	void SetUseTLS(bool val) { m_useTLS = val; }
 	bool UseTLS() const { return m_useTLS; }
+	void SetTLSAddress(const H323TransportAddress & addr) { m_tlsAddress = addr; }
+	H323TransportAddress GetTLSAddress() const;
+	void SetUseIPSec(bool val) { m_useIPSec = val; }
+	bool UseIPSec() const { return m_useIPSec; }
+
+#ifdef HAS_LANGUAGE
+	bool SetAssignedLanguage(const H225_RegistrationRequest_language & rrqLang, H225_RegistrationConfirm_language & rcfLang);
+	bool SetAssignedLanguage(H225_LocationConfirm_language & lcfLang);
+#endif
+	void SetLanguages(const PStringList & languages) { m_languages = languages; }
+	const PStringList & GetLanguages() { return m_languages; }
+	PString GetDefaultLanguage();
+
+	bool AddCallingPartyToSourceAddress() const { return m_addCallingPartyToSourceAddress; }
 
 	// smart pointer for EndpointRec
 	typedef SmartPtr<EndpointRec> Ptr;
@@ -411,7 +425,11 @@ protected:
 	long m_bandwidth;	// bandwidth currently occupied by this endpoint
 	long m_maxBandwidth; // maximum bandwidth allowed for this endpoint
 	bool m_useTLS;
+	H323TransportAddress m_tlsAddress;
+	bool m_useIPSec;	// for H.460.22 negotiation
 	bool m_additiveRegistrant;
+	PStringList m_languages;  // languages the user of this endpoint supports
+	bool m_addCallingPartyToSourceAddress;	// per endpoint switch
 };
 
 typedef EndpointRec::Ptr endptr;
@@ -1271,9 +1289,9 @@ public:
 
 	// vendor info for calling and called party
 	void SetCallingVendor(const PString & vendor, const PString & version);
-	void GetCallingVendor(PString & vendor, PString & version) const;
+	bool GetCallingVendor(PString & vendor, PString & version) const;
 	void SetCalledVendor(const PString & vendor, const PString & version);
-	void GetCalledVendor(PString & vendor, PString & version) const;
+	bool GetCalledVendor(PString & vendor, PString & version) const;
 
 	/** @return
 	    Fixed destination address for the call (size 0 if not set).
@@ -1310,7 +1328,7 @@ public:
 	int GetNoRemainingRoutes() const;
 	bool DisableRetryChecks() const;
 
-	void SetCodec(const PString &codec);
+	void SetCodec(const PString & codec);
 	PString GetCodec() const;
 
 	void SetMediaOriginatingIp(const PIPSocket::Address &addr);
@@ -1354,6 +1372,10 @@ public:
 	void SetSessionMultiplexDestination(WORD session, void * openedBy, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
 #endif
 
+	// should we use TLS on the outgoing leg, incoming determined by port caller uses
+	bool ConnectWithTLS() const { return m_connectWithTLS || (m_Called && m_Called->UseTLS()); }	// per call dynamicly and config setting
+	void SetConnectWithTLS(bool val) { m_connectWithTLS = val; }
+
 #ifdef HAS_H235_MEDIA
     typedef NATType EncDir;
     H235Authenticators & GetAuthenticators() { return m_authenticators; }
@@ -1578,6 +1600,8 @@ private:
 	PString m_callerID;	// forced caller ID or empty
 	PMutex m_portListMutex;
 	list<DynamicPort> m_dynamicPorts;
+	// should we use TLS on the outgoing leg, incoming determined by port caller uses
+	bool m_connectWithTLS;
 
 #ifdef HAS_H235_MEDIA
     H235Authenticators m_authenticators;
@@ -1688,7 +1712,7 @@ private:
 	void InternalRemove(iterator);
 	void InternalRemoveFailedLeg(iterator);
 
-	void InternalStatistics(unsigned & n, unsigned & act, unsigned & nb, unsigned & np, PString & msg, bool verbose) const;
+	void InternalStatistics(unsigned & n, unsigned & act, unsigned & nb, unsigned & np, unsigned & npr, PString & msg, bool verbose) const;
 
 	std::list<CallRec *> CallList;
 	std::list<CallRec *> RemovedList;
@@ -1704,7 +1728,8 @@ private:
 	long m_maximumBandwidthPerCall;	// maximum bandwidth allowed per call (<= 0 means unlimited)
 
 	// statistics
-	unsigned m_CallCount, m_successCall, m_neighborCall, m_parentCall, m_activeCall;
+	unsigned m_CallCount, m_successCall, m_neighborCall, m_parentCall, m_activeCall, m_proxiedCall, m_peakCall;
+	PTime m_peakTime;
 
 	/// timeout for a Connect message to be received
 	/// and for a signaling channel to be opened after ACF/ARQ
@@ -2089,7 +2114,7 @@ inline unsigned CallRec::GetDisconnectCause() const
 inline void CallRec::SetDisconnectCause( unsigned causeCode )
 {
 	// set the cause only if it has not been already set
-	if( m_disconnectCause == 0 )
+	if (m_disconnectCause == 0)
 		m_disconnectCause = causeCode;
 }
 
