@@ -51,7 +51,7 @@ const unsigned DEFAULT_ROUTE_PRIORITY = 1;
 const long DEFAULT_ROUTE_REQUEST_TIMEOUT = 10;
 const char* const CTIsection = "CTI::Agents";
 
-Route::Route() : m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_priority(DEFAULT_ROUTE_PRIORITY)
+Route::Route() : m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_useTLS(false), m_priority(DEFAULT_ROUTE_PRIORITY)
 {
 	m_destAddr.SetTag(H225_TransportAddress::e_nonStandardAddress);	// set to an invalid address
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
@@ -62,7 +62,7 @@ Route::Route(
 	const endptr & destEndpoint,
 	unsigned priority
 	) : m_destAddr(destEndpoint ? destEndpoint->GetCallSignalAddress() : H225_TransportAddress()), m_destEndpoint(destEndpoint), m_policy(policyName),
-	m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_priority(priority)
+	m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_useTLS(false), m_priority(priority)
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 	if (!destEndpoint) {
@@ -75,7 +75,7 @@ Route::Route(
 	const PString & policyName,
 	const H225_TransportAddress & destAddr,
 	unsigned priority
-	) : m_destAddr(destAddr), m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_priority(priority)
+	) : m_destAddr(destAddr), m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_useTLS(false), m_priority(priority)
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
@@ -86,7 +86,7 @@ Route::Route(
 	WORD destPort,
 	unsigned priority
 	) : m_destAddr(SocketToH225TransportAddr(destIpAddr, destPort)),
-	m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_priority(priority)
+	m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_useTLS(false), m_priority(priority)
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
@@ -106,6 +106,21 @@ bool Route::IsFailoverActive(
 	return m_rerouteCauses[cause >> 3] & (1UL << (cause & 7));
 }
 
+bool Route::SetLanguages(const PStringList & local, const PStringList & remote)
+{
+	if (local.GetSize() == 0 && remote.GetSize() == 0)
+		return true;
+
+	for (PINDEX i=0; i<local.GetSize(); ++i) {
+		for (PINDEX j =0; j<remote.GetSize(); ++j) {
+			if (local[i] == remote[j])
+				m_language.AppendString(local[i]);
+		}
+	}
+	return (m_language.GetSize() > 0);
+}
+
+
 // class RoutingRequest
 RoutingRequest::RoutingRequest()
 	: m_reason(-1), m_flags(0)
@@ -160,7 +175,7 @@ void RoutingRequest::RemoveAllRoutes()
 	m_routes.clear();
 }
 
-bool RoutingRequest::GetGatewayDestination(H225_TransportAddress & gw ) const 
+bool RoutingRequest::GetGatewayDestination(H225_TransportAddress & gw) const
 {
 	PIPSocket::Address addr;
 	if (!GetIPFromTransportAddr(m_gwDestination, addr)||!addr.IsValid())
@@ -170,6 +185,15 @@ bool RoutingRequest::GetGatewayDestination(H225_TransportAddress & gw ) const
 	return true;
 }
 
+bool RoutingRequest::SupportLanguages() const
+{
+#ifdef HAS_LANGUAGE
+	return GkConfig()->GetBoolean("RasSrv::LRQFeatures", "EnableLanguageRouting", false);
+#else
+	return false;
+#endif
+}
+
 // class AdmissionRequest
 template<> H225_ArrayOf_AliasAddress *AdmissionRequest::GetAliases()
 {
@@ -227,9 +251,9 @@ template<> const H225_TransportAddress *FacilityRequest::GetDestIP() const
 		? &m_request.m_alternativeAddress : NULL;
 }
 
-bool Policy::Handle(SetupRequest& request)
+bool Policy::Handle(SetupRequest & request)
 {
-	if( IsActive() ) {
+	if (IsActive()) {
 		const PString tagname = request.GetWrapper()->GetTagName();
 		const unsigned crv = request.GetWrapper()->GetCallReference();
 		PTRACE(5, "ROUTING\tChecking policy " << m_name
@@ -245,7 +269,7 @@ bool Policy::Handle(SetupRequest& request)
 
 bool Policy::Handle(FacilityRequest& request)
 {
-	if( IsActive() ) {
+	if (IsActive()) {
 		const PString tagname = request.GetWrapper()->GetTagName();
 		const unsigned crv = request.GetWrapper()->GetCallReference();
 		PTRACE(5, "ROUTING\tChecking policy " << m_name
@@ -264,7 +288,8 @@ void Policy::SetInstance(const PString & instance)
 {
 	PString policyName = PString(m_name); 
 	if (!instance.IsEmpty()) {
-		policyName = policyName + "::" + instance;
+		m_instance = instance;
+		policyName = policyName + "::" + m_instance;
 		m_name = *(PString *)policyName.Clone();
 	}
 	m_iniSection = "Routing::" + policyName;
@@ -355,7 +380,7 @@ bool Analyzer::Parse(SetupRequest & request)
 {
 	ReadLock lock(m_reloadMutex);
 	request.SetRejectReason(H225_ReleaseCompleteReason::e_calledPartyNotRegistered);
-	Policy *policy = ChoosePolicy(request.GetAliases(), m_rules[2]);
+	Policy * policy = ChoosePolicy(request.GetAliases(), m_rules[2]);
 	bool policyApplied = policy ? policy->Handle(request) : false;
 	if (!policyApplied && request.HasRoutes()) {
 		Route fallback;
@@ -398,16 +423,15 @@ Policy *Analyzer::ChoosePolicy(const H225_ArrayOf_AliasAddress *aliases, Rules &
 
 	// use rules.begin() as the default policy
 	// since "*" has the minimum key value
-	Rules::iterator iter, biter, eiter;
-	iter = biter = rules.begin(), eiter = rules.end();
+	Rules::iterator iter = rules.begin();
 	if (aliases && aliases->GetSize() > 0) {
 		for (PINDEX i = 0; i < aliases->GetSize(); ++i) {
 			const H225_AliasAddress & alias = (*aliases)[i];
 			iter = rules.find(alias.GetTagName());
-			if (iter != eiter)
+			if (iter != rules.end())
 				break;
 			PString destination(AsString(alias, false));
-			while (iter != biter) {
+			while (iter != rules.begin()) {
 				--iter; // search in reverse order
 				if (MatchPrefix(destination, iter->first) > 0)
 					return iter->second;
@@ -584,7 +608,7 @@ bool ExplicitPolicy::OnRequest(AdmissionRequest & request)
 
 bool ExplicitPolicy::OnRequest(SetupRequest & request)
 {
-	H225_Setup_UUIE &setup = request.GetRequest();
+	H225_Setup_UUIE & setup = request.GetRequest();
 	if (setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress)) {
 		// don't map IP here, for Setup already done in OnSetup()
 		Route route(m_name, setup.m_destCallSignalAddress);
@@ -636,10 +660,7 @@ bool InternalPolicy::OnRequest(SetupRequest & request)
 	return true;
 }
 
-bool InternalPolicy::FindByAliases(
-	RoutingRequest &request, 
-	H225_ArrayOf_AliasAddress &aliases
-	)
+bool InternalPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
 	list<Route> routes;
 	RegistrationTable::Instance()->FindEndpoint(aliases, roundRobin, true, routes);
@@ -651,10 +672,7 @@ bool InternalPolicy::FindByAliases(
 	return !routes.empty();
 }
 
-bool InternalPolicy::FindByAliases(
-	LocationRequest& request,
-	H225_ArrayOf_AliasAddress & aliases
-	)
+bool InternalPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
 	// do not apply round robin selection for Location ReQuests
 	list<Route> routes;
@@ -669,15 +687,12 @@ bool InternalPolicy::FindByAliases(
 	return !routes.empty();
 }
 
-bool InternalPolicy::FindByAliases(
-	SetupRequest& request,
-	H225_ArrayOf_AliasAddress & aliases
-	)
+bool InternalPolicy::FindByAliases(SetupRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
 	list<Route> routes;
 	if (RegistrationTable::Instance()->FindEndpoint(aliases, roundRobin, true, routes))
 		request.SetRejectReason(H225_ReleaseCompleteReason::e_gatewayResources);
-		
+
 	list<Route>::iterator i = routes.begin();
 	while (i != routes.end()) {
 		i->m_policy = m_name;
@@ -686,19 +701,25 @@ bool InternalPolicy::FindByAliases(
 	return !routes.empty();
 }
 
-bool InternalPolicy::FindByAliases(
-	AdmissionRequest& request,
-	H225_ArrayOf_AliasAddress & aliases
-	)
+bool InternalPolicy::FindByAliases(AdmissionRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
 	list<Route> routes;
 	if (RegistrationTable::Instance()->FindEndpoint(aliases, roundRobin, true, routes))
 		request.SetRejectReason(H225_AdmissionRejectReason::e_resourceUnavailable);
+
+	endptr ep;
+	if (request.SupportLanguages())
+		ep = RegistrationTable::Instance()->FindByEndpointId(request.GetRequest().m_endpointIdentifier);
 		
 	list<Route>::iterator i = routes.begin();
 	while (i != routes.end()) {
-		i->m_policy = m_name;
-		request.AddRoute(*i++);
+		if (ep && i->m_destEndpoint && !i->SetLanguages(i->m_destEndpoint->GetLanguages(), ep->GetLanguages())) {
+			PTRACE(4, m_name << "\tRoute found but rejected as no common language");
+			i++;
+		} else {
+			i->m_policy = m_name;
+			request.AddRoute(*i++);
+		}
 	}
 	return !routes.empty();
 }
@@ -772,8 +793,13 @@ bool DNSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 			continue;
 		PString alias(AsString(aliases[i], FALSE));
 		PINDEX at = alias.Find('@');
+		PINDEX hashhash = alias.Find("##");	// Polycom's domain##alias notation
 
-		PString domain = (at != P_MAX_INDEX) ? alias.Mid(at + 1) : alias;
+		PString domain = alias;
+		if (at != P_MAX_INDEX)
+			domain = alias.Mid(at + 1);
+		if (hashhash != P_MAX_INDEX)
+			domain = alias.Left(hashhash);
 		if (domain.Find("ip$") == 0)
 			domain.Replace("ip$", "", false);
 		PStringArray parts = SplitIPAndPort(domain, GK_DEF_ENDPOINT_SIGNAL_PORT);
@@ -784,15 +810,28 @@ bool DNSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 			H225_TransportAddress dest;
 			if (!request.GetGatewayDestination(dest)) {
 				dest = SocketToH225TransportAddr(addr, port);
-				H323SetAliasAddress(alias.Left(at), aliases[i]);
-				PTRACE(4, "ROUTING\tDNS policy resolves to " << alias.Left(at) << " @ " << AsDotString(dest));
+				PString aliasPart = "";
+				if (at != P_MAX_INDEX)
+					aliasPart = alias.Left(at);
+				if (hashhash != P_MAX_INDEX)
+					aliasPart = alias.Mid(hashhash + 2);
+				PTRACE(0, "JW alias=" << aliasPart);
+				H323SetAliasAddress(aliasPart, aliases[i]);
+				PTRACE(4, "ROUTING\tDNS policy resolves to " << aliasPart << " @ " << AsDotString(dest));
 			}
 
-			if (Toolkit::Instance()->IsGKHome(addr)) {
+			if (Toolkit::Instance()->IsGKHome(addr)
+				&& ( (port == GkConfig()->GetInteger("RoutedMode", "CallSignalPort", GK_DEF_CALL_SIGNAL_PORT))
+					|| (port == GkConfig()->GetInteger("RoutedMode", "TLSCallSignalPort", GK_DEF_TLS_CALL_SIGNAL_PORT))) ) {
 				// check if the domain is my IP, if so route to local endpoint if available
 				H225_ArrayOf_AliasAddress find_aliases;
 				find_aliases.SetSize(1);
-				H323SetAliasAddress(alias.Left(at), find_aliases[0]);
+				PString aliasPart = "";
+				if (at != P_MAX_INDEX)
+					aliasPart = alias.Left(at);
+				if (hashhash != P_MAX_INDEX)
+					aliasPart = alias.Mid(hashhash + 2);
+				H323SetAliasAddress(aliasPart, find_aliases[0]);
 				endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
 				if (ep) {
 					dest = ep->GetCallSignalAddress();
@@ -818,8 +857,13 @@ bool DNSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddre
 			continue;
 		PString alias(AsString(aliases[i], FALSE));
 		PINDEX at = alias.Find('@');
+		PINDEX hashhash = alias.Find("##");	// Polycom's domain##alias notation
 
-		PString domain = (at != P_MAX_INDEX) ? alias.Mid(at + 1) : alias;
+		PString domain = alias;
+		if (at != P_MAX_INDEX)
+			domain = alias.Mid(at + 1);
+		if (hashhash != P_MAX_INDEX)
+			domain = alias.Left(hashhash);
 		if (domain.Find("ip$") == 0)
 			domain.Replace("ip$", "", false);
 		PStringArray parts = SplitIPAndPort(domain, GK_DEF_ENDPOINT_SIGNAL_PORT);
@@ -831,8 +875,13 @@ bool DNSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddre
 			if (Toolkit::Instance()->IsGKHome(addr)) {
 				// only apply DNS policy to LRQs that resolve locally
 				H225_ArrayOf_AliasAddress find_aliases;
+				PString aliasPart = "";
+				if (at != P_MAX_INDEX)
+					aliasPart = alias.Left(at);
+				if (hashhash != P_MAX_INDEX)
+					aliasPart = alias.Mid(hashhash + 2);
 				find_aliases.SetSize(1);
-				H323SetAliasAddress(alias.Left(at), find_aliases[0]);
+				H323SetAliasAddress(aliasPart, find_aliases[0]);
 				endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
 				if (ep) {
 					if (!(RasServer::Instance()->IsGKRouted())) {
@@ -864,9 +913,7 @@ bool DNSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddre
 
 
 VirtualQueue::VirtualQueue()
-	:
-	m_active(false),
-	m_requestTimeout(DEFAULT_ROUTE_REQUEST_TIMEOUT*1000)
+	: m_active(false), m_requestTimeout(DEFAULT_ROUTE_REQUEST_TIMEOUT*1000)
 {
 }
 
@@ -884,11 +931,11 @@ VirtualQueue::~VirtualQueue()
 		RouteRequest *r = *i++;
 		r->m_sync.Signal();
 	}
-	
+
 	m_listMutex.Signal();
 
 	// wait a moment to give a chance to pending requests to cleanup
-	if( numrequests ) {
+	if (numrequests) {
 		PThread::Sleep(500);
 	}
 }
@@ -898,9 +945,8 @@ void VirtualQueue::OnReload()
 	PWaitAndSignal lock(m_listMutex);
 	
 	m_requestTimeout = GkConfig()->GetInteger(
-		CTIsection, 
-		GkConfig()->HasKey(CTIsection, "RequestTimeout")
-			? "RequestTimeout" : "CTI_Timeout", 
+		CTIsection,
+		GkConfig()->HasKey(CTIsection, "RequestTimeout") ? "RequestTimeout" : "CTI_Timeout", 
 		DEFAULT_ROUTE_REQUEST_TIMEOUT
 		) * 1000;
 	m_requestTimeout = PMAX((long)100,m_requestTimeout);	// min wait: 100 msec
@@ -913,7 +959,7 @@ void VirtualQueue::OnReload()
 		vqueues = GkConfig()->GetString(CTIsection, "VirtualQueue", "");
 	if( !vqueues.IsEmpty() ) {
 		m_virtualQueueAliases = vqueues.Tokenise(" ,;\t", false);
-		if( m_virtualQueueAliases.GetSize() > 0 ) {
+		if (m_virtualQueueAliases.GetSize() > 0) {
 			PTRACE(2, "VQueue\t(CTI) Virtual queues enabled (aliases:" << vqueues
 				<< "), request timeout: " << m_requestTimeout/1000 << " s");
 			m_active = true;
@@ -924,7 +970,7 @@ void VirtualQueue::OnReload()
 	vqueues = GkConfig()->GetString(CTIsection, "VirtualQueuePrefixes", "");
 	if( !vqueues.IsEmpty() ) {
 		m_virtualQueuePrefixes = vqueues.Tokenise(" ,;\t", false);
-		if( m_virtualQueuePrefixes.GetSize() > 0 ) {
+		if (m_virtualQueuePrefixes.GetSize() > 0) {
 			PTRACE(2, "VQueue\t(CTI) Virtual queues enabled (prefixes:" << vqueues
 				<< "), request timeout: " << m_requestTimeout/1000 << " s");
 			m_active = true;
@@ -952,39 +998,39 @@ void VirtualQueue::OnReload()
 
 bool VirtualQueue::SendRouteRequest(
 	/// source IP of the request (endpoint for ARQ, gatekeeper for LRQ)
-	const PString& source,
+	const PString & source,
 	/// calling endpoint
-	const PString& epid,
+	const PString & epid,
 	/// CRV (Call Reference Value) of the call associated with this request
 	unsigned crv,
 	/// destination (virtual queue) aliases as specified
 	/// by the calling endpoint (modified by this function on successful return)
-	H225_ArrayOf_AliasAddress* destinationInfo,
+	H225_ArrayOf_AliasAddress * destinationInfo,
 	/// destination (virtual queue) aliases as specified
 	/// by the calling endpoint (modified by this function on successful return)
-	PString* callSigAdr,
+	PString * callSigAdr,
 	/// bind IP for BindAndRouteToGateway
-	PString* bindIP,
+	PString * bindIP,
 	/// caller ID
-	PString* callerID,
+	PString * callerID,
 	/// should the call be rejected modified by this function on return)
 	bool & reject,
 	/// actual virtual queue name (should be present in destinationInfo too)
-	const PString& vqueue,
+	const PString & vqueue,
 	/// a sequence of aliases for the calling endpoint
 	/// (in the "alias:type[=alias:type]..." format)
-	const PString& sourceInfo,
+	const PString & sourceInfo,
 	/// the callID as string
-	const PString& callID,
+	const PString & callID,
 	/// the called IP for unregistered calls
-	const PString& calledip,
+	const PString & calledip,
 	/// vendor string of caller
-	const PString& vendorString
+	const PString & vendorString
 	)
 {
 	bool result = false;
 	bool duprequest = false;
-	if (RouteRequest *r = InsertRequest(epid, crv, callID, destinationInfo, callSigAdr, bindIP, callerID, duprequest)) {
+	if (RouteRequest * r = InsertRequest(epid, crv, callID, destinationInfo, callSigAdr, bindIP, callerID, duprequest)) {
 		PString cid = callID;
 		cid.Replace(" ", "-", true);
 		PString msg = "RouteRequest|" + source
@@ -1020,20 +1066,19 @@ bool VirtualQueue::SendRouteRequest(
 }
 
 bool VirtualQueue::IsDestinationVirtualQueue(
-	const PString& destinationAlias /// alias to be matched
+	const PString & destinationAlias /// alias to be matched
 	) const
 {
 	PWaitAndSignal lock(m_listMutex);
-	PINDEX i;
-	for (i = 0; i < m_virtualQueueAliases.GetSize(); ++i)
+	for (PINDEX i = 0; i < m_virtualQueueAliases.GetSize(); ++i)
 		if (m_virtualQueueAliases[i] == destinationAlias)
 			return true;
-	for (i = 0; i < m_virtualQueuePrefixes.GetSize(); ++i)
+	for (PINDEX i = 0; i < m_virtualQueuePrefixes.GetSize(); ++i)
 		if (destinationAlias.Find(m_virtualQueuePrefixes[i]) == 0)
 			return true;
 	
 	return (!m_virtualQueueRegex.IsEmpty())
-		&& Toolkit::MatchRegex(destinationAlias,m_virtualQueueRegex);
+		&& Toolkit::MatchRegex(destinationAlias, m_virtualQueueRegex);
 }
 
 bool VirtualQueue::RouteToAlias(
@@ -1108,19 +1153,19 @@ bool VirtualQueue::RouteToAlias(
 bool VirtualQueue::RouteToAlias(
 	/// alias for the routing target that
 	/// will replace the original destination info
-	const PString& targetAlias, 
+	const PString & targetAlias, 
 	/// will replace the original destinationCallSignallAddress
-	const PString& destinationIp, 
+	const PString & destinationIp, 
 	/// identifier of the endpoint associated with the route request
-	const PString& callingEpId, 
+	const PString & callingEpId, 
 	/// CRV for the call associated with the route request
 	unsigned crv,
 	/// callID of the call associated with the route request
-	const PString& callID,
+	const PString & callID,
 	// outgoing IP or empty
-	const PString& bindIP,
+	const PString & bindIP,
 	// callerID or empty
-	const PString& callerID,
+	const PString & callerID,
 	/// should this call be rejected
 	bool reject
 	)
@@ -1135,11 +1180,11 @@ bool VirtualQueue::RouteToAlias(
 
 bool VirtualQueue::RouteReject(
 	/// identifier of the endpoint associated with the route request
-	const PString& callingEpId, 
+	const PString & callingEpId, 
 	/// CRV of the call associated with the route request
 	unsigned crv,
 	/// callID of the call associated with the route request
-	const PString& callID
+	const PString & callID
 	)
 {
 	H225_ArrayOf_AliasAddress nullAgent;
@@ -1148,24 +1193,24 @@ bool VirtualQueue::RouteReject(
 
 VirtualQueue::RouteRequest* VirtualQueue::InsertRequest(
 	/// identifier for the endpoint associated with this request
-	const PString& callingEpId,
+	const PString & callingEpId,
 	/// CRV for the call associated with this request
 	unsigned crv,
 	/// callID for the call associated with this request
-	const PString& callID,
+	const PString & callID,
 	/// a pointer to an array to be filled with agent aliases
 	/// when the routing decision has been made
-	H225_ArrayOf_AliasAddress* agent,
+	H225_ArrayOf_AliasAddress * agent,
 	/// a pointer to a string  to be filled with a destinationCallSignalAddress
 	/// when the routing decision has been made (optional)
-	PString* callSigAdr,
+	PString * callSigAdr,
 	/// bind IP for BindAndRouteToGateway
-	PString* bindIP,
+	PString * bindIP,
 	/// caller ID
-	PString* callerID,
+	PString * callerID,
 	/// set by the function to true if another route request for the same
 	/// call is pending
-	bool& duplicate
+	bool & duplicate
 	)
 {
 	duplicate = false;
@@ -1250,7 +1295,7 @@ bool VirtualQueuePolicy::OnRequest(AdmissionRequest & request)
 					arq.IncludeOptionalField(H225_AdmissionRequest::e_destCallSignalAddress);
 				}
 				PStringArray adr_parts = SplitIPAndPort(*callSigAdr, GK_DEF_ENDPOINT_SIGNAL_PORT);
-				PString ip = adr_parts[0];
+				PIPSocket::Address ip(adr_parts[0]);
 				WORD port = (WORD)(adr_parts[1].AsInteger());
 				arq.m_destCallSignalAddress = SocketToH225TransportAddr(ip, port);
 			}
@@ -1313,7 +1358,7 @@ bool VirtualQueuePolicy::OnRequest(LocationRequest & request)
 			if (!reject && !callSigAdr->IsEmpty()) {
 				// 'explicit' policy can't handle LRQs, so we do it directly
 				PStringArray adr_parts = SplitIPAndPort(*callSigAdr, GK_DEF_ENDPOINT_SIGNAL_PORT);
-				PString ip = adr_parts[0];
+				PIPSocket::Address ip(adr_parts[0]);
 				WORD port = (WORD)(adr_parts[1].AsInteger());
 				Route route("vqueue", SocketToH225TransportAddr(ip, port));
 				route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
@@ -1403,7 +1448,7 @@ bool VirtualQueuePolicy::OnRequest(SetupRequest & request)
 				setup.IncludeOptionalField(H225_Setup_UUIE::e_destCallSignalAddress);
 			}
 			PStringArray adr_parts = SplitIPAndPort(*callSigAdr, GK_DEF_ENDPOINT_SIGNAL_PORT);
-			PString ip = adr_parts[0];
+			PIPSocket::Address ip(adr_parts[0]);
 			WORD port = (WORD)(adr_parts[1].AsInteger());
 			setup.m_destCallSignalAddress = SocketToH225TransportAddr(ip, port);
 		}
@@ -1556,6 +1601,11 @@ void ENUMPolicy::LoadConfig(const PString & instance)
 
 bool ENUMPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
+	PString service = request.GetServiceType();
+	if (!service && (service != m_instance)) {
+		PTRACE(4, "ROUTING\tPolicy " << m_name << " not supported for service " << service);
+		return false;
+	}
 
 #if P_DNS
 	for (PINDEX i = 0; i < m_enum_schema.GetSize(); ++i) {
@@ -1596,7 +1646,6 @@ bool ENUMPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddre
 				PTRACE(4, "ROUTING\tPolicy " << m_name << " " << enum_schema << " store destination for " << alias << " to " << AsString(dest));
 				request.SetGatewayDestination(dest);
 			}
-			changed = false;
 			return false;
 		}
 	}
@@ -1713,9 +1762,10 @@ bool DynamicPolicy::OnRequest(AdmissionRequest & request)
 		PString callid = AsString(arq.m_callIdentifier.m_guid);
 		PString messageType = "ARQ";
 		PString clientauthid = request.GetClientAuthId();
+		PString language = ep->GetDefaultLanguage();
 		DestinationRoutes destination;
 
-		RunPolicy(	/* in */ source, calledAlias, calledIP, caller, callingStationId, callid, messageType, clientauthid,
+		RunPolicy(	/* in */ source, calledAlias, calledIP, caller, callingStationId, callid, messageType, clientauthid, language,
 						/* out: */ destination);
 
 		if (destination.m_routes.empty() && !ResolveRoute(request,destination))
@@ -1772,9 +1822,14 @@ bool DynamicPolicy::OnRequest(LocationRequest & request)
 	PString callid = "";	/* not available for LRQs */
 	PString messageType = "LRQ";
 	PString clientauthid = "";	/* not available for LRQs */
+	PString language = "";
+#ifdef HAS_LANGUAGE
+	if (lrq.HasOptionalField(H225_LocationRequest::e_language) && (lrq.m_language.GetSize() > 0))
+		language = lrq.m_language[0];
+#endif
 	DestinationRoutes destination;
 
-	RunPolicy(	/* in */ source, calledAlias, calledIP, caller, callingStationId,callid, messageType, clientauthid,
+	RunPolicy(	/* in */ source, calledAlias, calledIP, caller, callingStationId,callid, messageType, clientauthid, language,
 					/* out: */ destination);
 
 	if (destination.m_routes.empty() && !ResolveRoute(request,destination))
@@ -1824,9 +1879,14 @@ bool DynamicPolicy::OnRequest(SetupRequest & request)
 	PString callid = AsString(setup.m_callIdentifier.m_guid);
 	PString messageType = "Setup";
 	PString clientauthid = request.GetClientAuthId();
+	PString language = "";
+#ifdef HAS_LANGUAGE
+	if (setup.HasOptionalField(H225_Setup_UUIE::e_language) && (setup.m_language.GetSize() > 0))
+		language = setup.m_language[0];
+#endif
 	DestinationRoutes destination;
 
-	RunPolicy(	/* in */ source, calledAlias, calledIP, caller, callingStationId, callid, messageType, clientauthid,
+	RunPolicy(	/* in */ source, calledAlias, calledIP, caller, callingStationId, callid, messageType, clientauthid, language,
 					/* out: */ destination);
 
 	if (destination.m_routes.empty() && !ResolveRoute(request,destination))
@@ -1936,6 +1996,7 @@ void SqlPolicy::RunPolicy(
 		const PString & callid,
 		const PString & messageType,
 		const PString & clientauthid,
+		const PString & language,
 		/* out: */
 		DestinationRoutes & destination)
 {
@@ -1950,6 +2011,7 @@ void SqlPolicy::RunPolicy(
 	params["i"] = callid;
 	params["m"] = messageType;
 	params["client-auth-id"] = clientauthid;
+	params["language"] = language;
 	GkSQLResult* result = m_sqlConn->ExecuteQuery(m_query, params, m_timeout);
 	if (result == NULL) {
 		PTRACE(2, m_name << ": query failed - timeout or fatal error");
@@ -1985,11 +2047,12 @@ void SqlPolicy::RunPolicy(
 			do {
 				PString destinationIp = resultRow[0].first;
 				PStringArray adr_parts = SplitIPAndPort(destinationIp, GK_DEF_ENDPOINT_SIGNAL_PORT);
-				PString ip = adr_parts[0];
+				PIPSocket::Address ip(adr_parts[0]);
 				WORD port = (WORD)(adr_parts[1].AsInteger());
 
 				Route route("Sql", SocketToH225TransportAddr(ip, port));
 				route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
+				if (!language) route.m_language.AppendString(language);
 				destination.AddRoute(route);
 
 				row++;
@@ -2023,12 +2086,13 @@ void SqlPolicy::RunPolicy(
 				PString destinationAlias = resultRow[0].first;
 				PString destinationIp = resultRow[1].first;
 				PStringArray adr_parts = SplitIPAndPort(destinationIp, GK_DEF_ENDPOINT_SIGNAL_PORT);
-				PString ip = adr_parts[0];
+				PIPSocket::Address ip(adr_parts[0]);
 				WORD port = (WORD)(adr_parts[1].AsInteger());
 
 				Route route("Sql", SocketToH225TransportAddr(ip, port));
 				route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
 				route.m_destNumber = destinationAlias;
+				if (!language) route.m_language.AppendString(language);
 				destination.AddRoute(route);
 
 				row++;
@@ -2097,6 +2161,97 @@ bool CatchAllPolicy::CatchAllRoute(RoutingRequest & request) const
 }
 
 
+
+URIServicePolicy::URIServicePolicy()
+{
+	m_name = "URIService";
+	m_iniSection = "Routing::URIService";
+}
+
+void URIServicePolicy::LoadConfig(const PString & instance)
+{
+	PStringToString routes = GkConfig()->GetAllKeyValues(m_iniSection);
+	for (PINDEX i = 0; i < routes.GetSize(); i++) {
+		PString service = routes.GetKeyAt(i);
+		PString gwDestination = routes.GetDataAt(i);
+		PStringArray parts = SplitIPAndPort(gwDestination, GK_DEF_ENDPOINT_SIGNAL_PORT);
+		PString dom = parts[0];
+		PIPSocket::Address addr;
+		if (PIPSocket::GetHostAddress(dom, addr)) 
+			dom = addr.AsString();
+		WORD port = (WORD)parts[1].AsUnsigned();
+		H225_TransportAddress dest;
+		if (!GetTransportAddress(dom, port, dest)) {
+			PTRACE(4, "ROUTING\tPolicy " << m_name << " " << service << " Could not resolve " << gwDestination);
+			continue;
+		}
+		m_uriServiceRoute.insert(make_pair(service, dest));
+	}
+}
+
+bool URIServicePolicy::OnRequest(AdmissionRequest & request) 
+{ 
+    return URIServiceRoute(request, request.GetAliases()); 
+}
+
+bool URIServicePolicy::OnRequest(LocationRequest & request) 
+{ 
+    return URIServiceRoute(request, request.GetAliases()); 
+}
+
+bool URIServicePolicy::OnRequest(SetupRequest & request) 
+{ 
+    return URIServiceRoute(request, request.GetAliases());
+}
+
+bool URIServicePolicy::URIServiceRoute(RoutingRequest & request, H225_ArrayOf_AliasAddress * aliases) const
+{
+	if (!aliases)
+		return false;
+
+	for (PINDEX a = 0; a < aliases->GetSize(); a++) {
+		PString alias = H323GetAliasAddressString((*aliases)[a]);
+		PINDEX colon = alias.Find(":");
+		if (colon == 0 || colon == P_MAX_INDEX)
+			continue;
+
+		PString service = alias.Left(colon);
+		map<PString,H225_TransportAddress>::const_iterator i = m_uriServiceRoute.find(service);
+		if (i == m_uriServiceRoute.end())
+			continue;
+
+		H225_TransportAddress destination = i->second;
+		PString newAlias = alias.Mid(colon+1);
+		request.SetFlag(RoutingRequest::e_aliasesChanged);
+		H323SetAliasAddress(newAlias, (*aliases)[a]);
+
+		PString domain = newAlias;
+		PINDEX at = newAlias.Find("@");
+		if (at != P_MAX_INDEX)  // URL Schema
+			domain = newAlias.Mid(at+1);
+
+		if (!IsIPAddress(domain)   // Not an IP address
+			&& (domain.FindRegEx(PRegularExpression(":[0-9]+$", PRegularExpression::Extended)) == P_MAX_INDEX)) {
+			PTRACE(4, "ROUTING\tPolicy " << m_name << " " << service << " store destination for " << alias << " to " << AsString(destination));
+			request.SetServiceType(service);
+			request.SetGatewayDestination(destination);
+			return false;  // Fall through to the next 	
+		} 
+
+		// add a route and stop going any further
+		PTRACE(4, "ROUTING\tPolicy " << m_name << " " << service << " set destination for " << alias << " to " << AsString(destination));
+		Route * route = new Route(m_name, destination);
+		route->m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(destination);
+		request.AddRoute(*route);
+		delete route;
+		return true;
+	}
+
+	// No route found
+	return false;
+}
+
+
 namespace { // anonymous namespace
 	SimpleCreator<ExplicitPolicy> ExplicitPolicyCreator("explicit");
 	SimpleCreator<InternalPolicy> InternalPolicyCreator("internal");
@@ -2107,6 +2262,7 @@ namespace { // anonymous namespace
 	SimpleCreator<ENUMPolicy> ENUMPolicyCreator("enum");
 	SimpleCreator<SqlPolicy> SqlPolicyCreator("sql");
 	SimpleCreator<CatchAllPolicy> CatchAllPolicyCreator("catchall");
+	SimpleCreator<URIServicePolicy> URISevicePolicyCreator("uriservice");
 }
 
 
