@@ -15,7 +15,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef ROUTING_H
-#define ROUTING_H "@(#) $Id: Routing.h,v 1.81 2013/09/03 14:46:39 willamowius Exp $"
+#define ROUTING_H "@(#) $Id: Routing.h,v 1.89 2014/04/23 18:32:33 willamowius Exp $"
 
 #include <map>
 #include <list>
@@ -73,13 +73,15 @@ public:
 		unsigned priority = 1
 		);
 
-	bool operator< (const Route & rhs) { return m_priority < rhs.m_priority; }
+	bool operator< (const Route & rhs) const { return m_priority < rhs.m_priority; }
 
 	unsigned GetPriority() const { return m_priority; }
 	void SetPriority(unsigned p) { m_priority = p; }
 	PString AsString() const;
 	bool IsFailoverActive(unsigned cause) const;
 
+	bool SetLanguages(const PStringList & local, const PStringList & remote);
+
 	H225_TransportAddress m_destAddr; /// destination address for signaling
 	endptr m_destEndpoint; /// destination endpoint record (if available)
 	PString m_policy; /// name of the policy that found the route
@@ -88,6 +90,8 @@ public:
 	unsigned m_flags; /// additional route specific flags
 	PString m_destNumber; /// rewritten number (corresponds to Toolkit::RewriteE164)
 	PString m_destOutNumber; /// number actually sent to the called party  (corresponds to Toolkit::GWRewriteE164)
+	PStringList m_language;  /// Language tags to use with this route.
+	bool m_useTLS;
 
 protected:
 	unsigned char m_rerouteCauses[16]; /// bit flags to trigger rerouting on particular Q931 causes
@@ -125,6 +129,10 @@ public:
 	void SetGatewayDestination(const H225_TransportAddress & gw) { m_gwDestination = gw; }
 	bool GetGatewayDestination(H225_TransportAddress & gw ) const;
 
+	PString GetServiceType() const { return m_serviceType; }
+	void SetServiceType(const PString & service) { m_serviceType = service; }
+
+	bool SupportLanguages() const;
 private:
 	RoutingRequest(const RoutingRequest &);
 	RoutingRequest& operator=(const RoutingRequest &);
@@ -137,6 +145,7 @@ private:
 	PString m_sourceIP;
 	PString m_callerID;
 	H225_TransportAddress m_gwDestination;
+	PString m_serviceType;
 };
 
 template<class R, class W>
@@ -154,12 +163,12 @@ public:
 	bool Process();
 
 	ReqObj & GetRequest() { return m_request; }
-	Wrapper *GetWrapper() { return m_wrapper; }
+	Wrapper * GetWrapper() { return m_wrapper; }
 	H225_ArrayOf_AliasAddress *GetAliases();
 	void SetAliases(H225_ArrayOf_AliasAddress & aliases);
 	const H225_TransportAddress * GetDestIP() const;
 	const ReqObj & GetRequest() const { return m_request; }
-	const Wrapper *GetWrapper() const { return m_wrapper; }
+	const Wrapper * GetWrapper() const { return m_wrapper; }
 	PString GetCallingStationId() const { return m_callingStationId; }
 	PUInt64 GetClientAuthId() const { return m_clientAuthId; }
 	const H225_ArrayOf_AliasAddress * GetAliases() const
@@ -266,6 +275,7 @@ protected:
 	/// of derived policies, default value is "undefined"
 	const char* m_name;
 	PString m_iniSection;
+	PString m_instance;
 };
 
 
@@ -469,6 +479,7 @@ protected:
 	unsigned int m_rejectReason;
 	bool m_aliasesChanged;
 	H225_ArrayOf_AliasAddress m_newAliases;
+	PStringList m_language;
 };
 
 // superclass for dynamic policies like sql and lua scripring
@@ -494,6 +505,7 @@ protected:
 		const PString & callid,
 		const PString & messageType,
 		const PString & clientauthid,
+		const PString & language,
 		/* out: */
 		DestinationRoutes & destination) = 0;
 
@@ -526,6 +538,7 @@ protected:
 		const PString & callid,
 		const PString & messageType,
 		const PString & clientauthid,
+		const PString & language,
 		/* out: */
 		DestinationRoutes & destination);
 
@@ -560,6 +573,24 @@ protected:
 
 
 
+class URIServicePolicy : public Policy {
+public:
+	URIServicePolicy();
+	virtual ~URIServicePolicy() { }
+
+protected:
+	virtual bool OnRequest(AdmissionRequest & request);
+	virtual bool OnRequest(LocationRequest & request);
+	virtual bool OnRequest(SetupRequest & request);
+
+	bool URIServiceRoute(RoutingRequest & request, H225_ArrayOf_AliasAddress * aliases) const;
+
+	virtual void LoadConfig(const PString & instance);
+	
+	map<PString,H225_TransportAddress> m_uriServiceRoute;
+};
+
+
 template<class R, class W>
 inline bool Request<R, W>::Process()
 {
