@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef TOOLKIT_H
-#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.114 2013/09/10 15:49:47 willamowius Exp $"
+#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.116 2013/10/20 19:43:20 shorne Exp $"
 
 #include <vector>
 #include <map>
@@ -234,6 +234,34 @@ class Toolkit : public Singleton<Toolkit>
 		RewriteData * m_Rewrite;
 	};
 
+	class VendorData {
+	public:
+		VendorData(PConfig *, const PString &);
+		~VendorData();
+		void AddSection(PConfig *config, const PString & section);
+		PINDEX Size() const { return m_size; }
+		const PString & Key(PINDEX i) const { return m_VendorKey[i]; }
+		const PString & Value(PINDEX i) const { return m_VendorValue[i]; }
+
+	private:
+		PString *m_VendorKey, *m_VendorValue;
+		PINDEX m_size;
+	};
+
+	class VendorModeTool {
+	public:
+		VendorModeTool() : m_vendorInfo(0) {}
+		~VendorModeTool() { delete m_vendorInfo; }
+		void LoadConfig(PConfig *);
+		int ModeSelection(const PString & str) const;
+
+	private:
+		VendorData * m_vendorInfo;
+	};
+
+	int SelectRoutingVendorMode(const PString & vendor) const
+	{ return m_venderMode.ModeSelection(vendor); }
+
 	class AssignedAliases {
 	 public:
 #if HAS_DATABASE
@@ -359,6 +387,34 @@ class Toolkit : public Singleton<Toolkit>
 	bool MatchHostCert(SSL * ssl, PIPSocket::Address addr);
 #endif
 
+
+#ifdef HAS_LANGUAGE
+	class AssignedLanguage {
+	 public:
+#if HAS_DATABASE
+	    AssignedLanguage();
+		~AssignedLanguage();
+
+		bool LoadSQL(PConfig *);
+		bool DatabaseLookup(const PString &, PStringArray &);
+#endif
+		bool QueryAssignedLanguage(const PString & alias, PStringArray & languages);
+		bool GetLanguage(const H225_ArrayOf_AliasAddress & alias, PStringArray & aliaslist);
+
+#if HAS_DATABASE
+	private:
+		bool m_sqlactive;
+		// connection to the SQL database
+		GkSQLConnection* m_sqlConn;
+		// parametrized query string for the auth condition string retrieval
+		PString m_query;
+		// query timeout
+		long m_timeout;
+#endif
+	};
+	AssignedLanguage GetAssignedLanguages() const { return m_assignedLanguage; }
+#endif
+
 	/// maybe modifies #alias#. returns true if it did
 	bool RewriteE164(H225_AliasAddress & alias);
 	bool RewriteE164(H225_ArrayOf_AliasAddress & aliases);
@@ -641,8 +697,12 @@ protected:
 #ifdef h323v6
 	AssignedGatekeepers m_AssignedGKs;
 #endif
+#ifdef HAS_LANGUAGE
+	AssignedLanguage m_assignedLanguage;
+#endif
+	VendorModeTool m_venderMode;
 #if HAS_DATABASE
-    AlternateGatekeepers m_AlternateGKs;
+	AlternateGatekeepers m_AlternateGKs;
 	QoSMonitor m_qosMonitor;
 #endif
 	RouteTable m_RouteTable;
