@@ -3,7 +3,7 @@
  *
  * Module for CLI/ANI manipulation.
  *
- * $Id: clirw.cxx,v 1.29 2013/04/10 11:13:50 willamowius Exp $
+ * $Id: clirw.cxx,v 1.31 2013/09/02 19:57:18 willamowius Exp $
  *
  * Copyright (c) 2005, Michal Zygmuntowicz
  * Copyright (c) 2007-2013, Jan Willamowius
@@ -106,10 +106,10 @@ struct RewriteRule_greater : public std::binary_function<CLIRewrite::RewriteRule
 
 	bool operator()(const CLIRewrite::RewriteRule &e1, const CLIRewrite::RewriteRule &e2) const
 	{
-		int diff = e1.m_matchType - e2.m_matchType;
+		long long int diff = e1.m_matchType - e2.m_matchType;
 		if (diff != 0)
 			return diff < 0;
-		 diff = e1.m_prefix.length() - e2.m_prefix.length();
+		diff = e1.m_prefix.length() - e2.m_prefix.length();
 		if (diff != 0)
 			return diff > 0;
 		return e1.m_prefix.compare(e2.m_prefix) > 0;
@@ -722,9 +722,11 @@ void CLIRewrite::Rewrite(
 					);
 			}
 			if (rule->m_rewriteType == RewriteRule::PrefixToPrefix
-					&& rule->m_matchType == RewriteRule::MatchCallerNumber)
-				newcli = RewriteString(cli, rule->m_prefix.c_str(), newcli);
-			
+					&& rule->m_matchType == RewriteRule::MatchCallerNumber) {
+				PString unused;
+				newcli = RewriteString(cli, rule->m_prefix.c_str(), newcli, unused);
+			}
+
 			PTRACE(5, "CLIRW\t" << (inbound ? "Inbound" : "Outbound")
 				<< " CLI rewrite to '" << newcli << "' by the rule " << rule->AsString()
 				);
