@@ -6,7 +6,7 @@ use strict;
 
 use IO::Socket;
 
-print "THIS IS NO REAL BILLING APPLICATION, JUST A DEMO HOW TO CONNECT TO THE GATEKEEPER.\nWRITE YOUR OWN CLIENT AND USE [StatusAcct] MESSAGES!\n\n";
+print "THIS IS NOT A REAL BILLING APPLICATION, JUST A DEMO OF HOW TO CONNECT TO THE GATEKEEPER.\nWRITE YOUR OWN CLIENT AND USE [StatusAcct] MESSAGES!\n\n";
 
 if (@ARGV != 1) {
 	print "usage: billing.pl <gatekeeper_host>\n";
