@@ -51,6 +51,7 @@ protected:
 		const PString & callid,
 		const PString & messageType,
 		const PString & clientauthid,
+		const PString & language,
 		/* out: */
 		DestinationRoutes & destination);
 
@@ -137,6 +138,7 @@ void ForwardingPolicy::RunPolicy(
 		const PString & callid,
 		const PString & messageType,
 		const PString & clientauthid,
+		const PString & language,
 		/* out: */
 		DestinationRoutes & destination)
 {
@@ -150,11 +152,12 @@ void ForwardingPolicy::RunPolicy(
 	params["i"] = callid;
 	params["m"] = messageType;
 	params["client-auth-id"] = clientauthid;
+	params["language"] = language;
 
 	// if IP called, check if its an internal EP and change calledAlias/calledIP
 	if (calledAlias.IsEmpty() && IsIPAddress(calledIP)) {
 		PStringArray adr_parts = calledIP.Tokenise(":", FALSE);
-		PString ip = adr_parts[0];
+		PIPSocket::Address ip(adr_parts[0]);
 		WORD port = (WORD)(adr_parts[1].AsInteger());
 		if (port == 0)
 			port = GK_DEF_ENDPOINT_SIGNAL_PORT;
@@ -172,7 +175,7 @@ void ForwardingPolicy::RunPolicy(
 	FindEPForwardingRules(params, 0, DEFAULT_PRIORITY, destination);
 
 	PTRACE(5, "Fwd\t" << destination.m_routes.size() << " routes added");
-	for(std::list<Route>::iterator it = destination.m_routes.begin(); it !=destination.m_routes.end(); ++it) {
+	for (std::list<Route>::iterator it = destination.m_routes.begin(); it !=destination.m_routes.end(); ++it) {
 		PTRACE(5, "Fwd\tRoute=" << it->AsString());
 	}
 	if (destination.ChangeAliases()) {
@@ -245,11 +248,10 @@ bool ForwardingPolicy::FindEPForwardingRules(
 					// set a route if forward to IP
 					PString destinationIp = forwardDestination;
 					PStringArray adr_parts = destinationIp.Tokenise(":", FALSE);
-					PString ip = adr_parts[0];
+					PIPSocket::Address ip(adr_parts[0]);
 					WORD port = (WORD)(adr_parts[1].AsInteger());
 					if (port == 0)
 						port = GK_DEF_ENDPOINT_SIGNAL_PORT;
-					//PTRACE(0, "JW forward destination to " << forwardDestination << " prio=" << priority + recursionDepth);
 					Route route("ForwardUnconditionalOrBusy", SocketToH225TransportAddr(ip, port), priority + recursionDepth);
 					route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
 					if ((forwardType == FORWARD_UNCONDITIONAL)
@@ -265,7 +267,6 @@ bool ForwardingPolicy::FindEPForwardingRules(
 					called.SetSize(1);
 					H323SetAliasAddress(params["c"], called[0]);
 					endptr ep = RegistrationTable::Instance()->FindByAliases(called);
-					//PTRACE(0, "JW call to " << AsString(called, false) << " ep=" << ep << " call=" << CallTable::Instance()->FindCallRec(ep));
 					if ((forwardType == FORWARD_UNCONDITIONAL)
 						|| (ep && CallTable::Instance()->FindCallRec(ep))) {
 						// check if new destination is also forwarded
@@ -273,17 +274,14 @@ bool ForwardingPolicy::FindEPForwardingRules(
 						params["p"] = "";
 						if (FindEPForwardingRules(params, recursionDepth+1, DEFAULT_PRIORITY, destination)) {
 							PTRACE(5, "Fwd\tSkipping forward to " << forwardDestination << " (also redirected uncond/busy)");
-							//PTRACE(0, "JW destination aliases=" << destination.GetNewAliases());
 						} else {
 							if (priority <= 1) {
 								// just rewrite the destination if forward to alias
-								//PTRACE(0, "JW rewriting destination to " << forwardDestination);
 								H225_ArrayOf_AliasAddress newAliases;
 								newAliases.SetSize(1);
 								H323SetAliasAddress(forwardDestination, newAliases[0]);
 								destination.SetNewAliases(newAliases);
 							} else {
-								//PTRACE(0, "JW forward destination to " << forwardDestination << " prio=" << priority);
 								// replace an NoAnswer or Error forward
 								if (ep) {
 									Route route("ForwardNoAnswerOrError", ep, priority + recursionDepth);
@@ -303,7 +301,7 @@ bool ForwardingPolicy::FindEPForwardingRules(
 					// set a route if forward to IP
 					PString destinationIp = forwardDestination;
 					PStringArray adr_parts = destinationIp.Tokenise(":", FALSE);
-					PString ip = adr_parts[0];
+					PIPSocket::Address ip(adr_parts[0]);
 					WORD port = (WORD)(adr_parts[1].AsInteger());
 					if (port == 0)
 						port = GK_DEF_ENDPOINT_SIGNAL_PORT;
@@ -316,7 +314,6 @@ bool ForwardingPolicy::FindEPForwardingRules(
 					forwardAliases.SetSize(1);
 					H323SetAliasAddress(forwardDestination, forwardAliases[0]);
 					endptr ep = RegistrationTable::Instance()->FindByAliases(forwardAliases);
-					//PTRACE(0, "JW forwarding NoAnswer to " << forwardDestination << " ep=" << ep);
 					// check if destination also has uncond/busy forwarding rules
 					params["c"] = forwardDestination;
 					params["p"] = "";
