@@ -2,7 +2,7 @@
 //
 // gk.cxx for GNU Gatekeeper
 //
-// Copyright (c) 2000-2013, Jan Willamowius
+// Copyright (c) 2000-2014, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -20,6 +20,7 @@
 #endif
 #ifdef P_LINUX
 #include <sys/resource.h>
+#include <sys/mman.h>
 #endif
 #include <h225.h>
 #include "h323util.h"
@@ -83,6 +84,17 @@ const char * KnownConfigEntries[][2] = {
 	{ "AssignedGatekeepers::SQL", "Password" },
 	{ "AssignedGatekeepers::SQL", "Query" },
 	{ "AssignedGatekeepers::SQL", "Username" },
+#ifdef HAS_LANGUAGE
+	{ "AssignedLanguage::SQL", "CacheTimeout" },
+	{ "AssignedLanguage::SQL", "Database" },
+	{ "AssignedLanguage::SQL", "Driver" },
+	{ "AssignedLanguage::SQL", "Host" },
+	{ "AssignedLanguage::SQL", "Library" },
+	{ "AssignedLanguage::SQL", "MinPoolSize" },
+	{ "AssignedLanguage::SQL", "Password" },
+	{ "AssignedLanguage::SQL", "Query" },
+	{ "AssignedLanguage::SQL", "Username" },
+#endif
 #endif
 	{ "CTI::Agents", "CTI_Timeout" },	// obsolete, but still accepted
 	{ "CTI::Agents", "RequestTimeout" },
@@ -95,7 +107,8 @@ const char * KnownConfigEntries[][2] = {
 	{ "CTI::MakeCall", "EndpointAlias" },
 	{ "CTI::MakeCall", "Gatekeeper" },
 	{ "CTI::MakeCall", "Interface" },
-	{ "CTI::MakeCall", "UseH450" },
+	{ "CTI::MakeCall", "UseH450" }, // obsolete, but still accepted
+	{ "CTI::MakeCall", "TransferMethod" },
 	{ "CallTable", "AcctUpdateInterval" },
 	{ "CallTable", "DefaultCallDurationLimit" },
 	{ "CallTable", "DefaultCallTimeout" },
@@ -272,6 +285,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "GkStatus::Filtering", "NewRCFOnly" },
 	{ "GkStatus::Message", "Compact" },
 	{ "GkStatus::Message", "RCF" },
+	{ "GkStatus::Message", "URQ" },
 	{ "LogFile", "Filename" },
 	{ "LogFile", "Rotate" },
 	{ "LogFile", "RotateDay" },
@@ -290,6 +304,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "PortNotifications", "StatusPortClose" },
 	{ "PortNotifications", "T120PortOpen" },
 	{ "PortNotifications", "T120PortClose" },
+	{ "Proxy", "CheckH46019KeepAlivePT" },
 	{ "Proxy", "DisableRTPQueueing" },
 	{ "Proxy", "Enable" },
 	{ "Proxy", "EnableRTCPStats" },
@@ -358,6 +373,9 @@ const char * KnownConfigEntries[][2] = {
 	{ "RasSrv::LRQFeatures", "AcceptNonNeighborLCF" },
 	{ "RasSrv::LRQFeatures", "AcceptNonNeighborLRQ" },
 	{ "RasSrv::LRQFeatures", "AlwaysForwardLRQ" },
+#ifdef HAS_LANGUAGE
+	{ "RasSrv::LRQFeatures", "EnableLanguageRouting" },
+#endif
 	{ "RasSrv::LRQFeatures", "ForwardHopCount" },
 	{ "RasSrv::LRQFeatures", "ForwardLRQ" },
 	{ "RasSrv::LRQFeatures", "ForwardResponse" },
@@ -393,6 +411,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "RewriteSourceAddress", "OnlyValid10Dand11D" },
 	{ "RewriteSourceAddress", "ReplaceChar" },
 	{ "RewriteSourceAddress", "Rules" },
+	{ "RewriteSourceAddress", "TreatNumberURIDialedDigits" },
 	{ "RoutedMode", "AcceptNeighborCalls" },
 	{ "RoutedMode", "AcceptNeighborsCalls" },
 	{ "RoutedMode", "AcceptUnregisteredCalls" },
@@ -431,13 +450,14 @@ const char * KnownConfigEntries[][2] = {
 	{ "RoutedMode", "EnableH460P" },
 #endif
 	{ "RoutedMode", "FailoverCauses" },
+	{ "RoutedMode", "FilterEmptyFacility" },
 	{ "RoutedMode", "ForceNATKeepAlive" },
 	{ "RoutedMode", "ForwardOnFacility" },
 	{ "RoutedMode", "GKRouted" },
 	{ "RoutedMode", "GenerateCallProceeding" },
 	{ "RoutedMode", "GkRouted" },
-#ifdef HAS_H235_MEDIA
-	{ "RoutedMode", "H235HalfCallMediaStrength" },
+#if defined(HAS_H235_MEDIA) && defined (HAS_SETTOKENLENGTH)
+	{ "RoutedMode", "H235HalfCallMaxTokenLength" },
 #endif
 	{ "RoutedMode", "H245PortRange" },
 	{ "RoutedMode", "H245Routed" },
@@ -451,8 +471,10 @@ const char * KnownConfigEntries[][2] = {
 	{ "RoutedMode", "H46023SignalGKRouted" },
 	{ "RoutedMode", "H46023STUN" },
 	{ "RoutedMode", "H46024ForceDirect" },
+	{ "RoutedMode", "H46023ForceNat" },
 #endif
 	{ "RoutedMode", "NATStdMin" },
+	{ "RoutedMode", "ProxyHandlerHighPrio" },
 	{ "RoutedMode", "Q931PortRange" },
 	{ "RoutedMode", "RDSservers" },
 	{ "RoutedMode", "RemoveCallOnDRQ" },
@@ -622,23 +644,30 @@ const char * KnownConfigEntries[][2] = {
 	{ "TLS", "CAFile" },
 	{ "TLS", "Certificates" },
 	{ "TLS", "CheckCertificateIP" },
+	{ "TLS", "CipherList" },
 	{ "TLS", "EnableTLS" },
 	{ "TLS", "Passphrase" },
 	{ "TLS", "PrivateKey" },
+#ifdef HAS_WEAK_TLS
+	{ "TLS", "RequireRemoteCertificate" },
+#endif
 #endif
 
 	// ignore name partially to check
+	{ "EP::", "AddCallingPartyToSourceAddress" },
 	{ "EP::", "AddNumbers" },
 	{ "EP::", "AdditionalDestinationAlias" },
 	{ "EP::", "CalledTypeOfNumber" },
 	{ "EP::", "CallingTypeOfNumber" },
 	{ "EP::", "Capacity" },
+	{ "EP::", "DisableCallCreditCapabilities" },
 #ifdef HAS_H46017
 	{ "EP::", "DisableH46017" },
 #endif
 #ifdef HAS_H46018
 	{ "EP::", "DisableH46018" },
 #endif
+	{ "EP::", "ForceGateway" },
 	{ "EP::", "GatewayPrefixes" },
 	{ "EP::", "GatewayPriority" },
 	{ "EP::", "MaxBandwidth" },
@@ -668,7 +697,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "Neighbor::", "SendPassword" },
 	{ "Neighbor::", "SendPrefixes" },
 #ifdef HAS_TLS
-	{ "Neighbor", "UseTLS" },
+	{ "Neighbor::", "UseTLS" },
 #endif
 
 	// uncheckable sections
@@ -681,6 +710,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "H350PasswordAuth", "*" },
 #endif
 	{ "ModeSelection", "*" },
+	{ "ModeVendorSelection", "*" },
 	{ "NATedEndpoints", "*" },
 	{ "PrefixAuth", "*" },
 	{ "RasSrv::AlternateGatekeeper", "*" },
@@ -697,6 +727,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "RewriteCLI", "*" },
 	{ "Routing::Explicit", "*" },
 	{ "Routing::NumberAnalysis", "*" },
+	{ "Routing::URIService", "*" },
 	{ "RoutingPolicy", "*" },
 	{ "RoutingPolicy::OnARQ", "*" },
 	{ "RoutingPolicy::OnFacility", "*" },
@@ -862,62 +893,56 @@ void ReloadHandler()
 	Gatekeeper::ReopenLogFile();
 
 	// only one thread must do this
-#ifdef hasNoMutexWillBlock
-	if (!ReloadMutex.Try())
-#else
-	if (ReloadMutex.WillBlock())
-#endif
-		return;
+	if (ReloadMutex.Wait(0)) {
+		/*
+		** Enter critical Section
+		*/
+		ConfigReloadMutex.StartWrite();
 
-	/*
-	** Enter critical Section
-	*/
-	PWaitAndSignal reload(ReloadMutex);
+		/*
+		** Force reloading config
+		*/
+		Toolkit::Instance()->ReloadConfig();
 
-	ConfigReloadMutex.StartWrite();
+		SoftPBX::TimeToLive = GkConfig()->GetInteger("TimeToLive", SoftPBX::TimeToLive);
 
-	/*
-	** Force reloading config
-	*/
-	Toolkit::Instance()->ReloadConfig();
+		/*
+		** Update all gateway prefixes
+		*/
 
-	SoftPBX::TimeToLive = GkConfig()->GetInteger("TimeToLive", SoftPBX::TimeToLive);
+		CallTable::Instance()->LoadConfig();
+		RegistrationTable::Instance()->LoadConfig();
+		CallTable::Instance()->UpdatePrefixCapacityCounters();
 
-	/*
-	** Update all gateway prefixes
-	*/
+		// don't put this in LoadConfig()
+		RasServer::Instance()->SetRoutedMode();
 
-	CallTable::Instance()->LoadConfig();
-	RegistrationTable::Instance()->LoadConfig();
-	CallTable::Instance()->UpdatePrefixCapacityCounters();
+		// Load ENUM servers
+		RasServer::Instance()->SetENUMServers();
 
-	// don't put this in LoadConfig()
-	RasServer::Instance()->SetRoutedMode();
+		// Load RDS servers
+		RasServer::Instance()->SetRDSServers();
 
-	// Load ENUM servers
-	RasServer::Instance()->SetENUMServers();
+		RasServer::Instance()->LoadConfig();
 
-	// Load RDS servers
-	RasServer::Instance()->SetRDSServers();
+		Gatekeeper::EnableLogFileRotation();
 
-	RasServer::Instance()->LoadConfig();
+		ConfigReloadMutex.EndWrite();
 
-	Gatekeeper::EnableLogFileRotation();
+		/*
+		** Don't disengage current calls!
+		*/
+		PTRACE(3, "GK\tCarry on current calls.");
 
-	ConfigReloadMutex.EndWrite();
+		SNMP_TRAP(3, SNMPInfo, General, "Full config reloaded");
 
-	/*
-	** Don't disengage current calls!
-	*/
-	PTRACE(3, "GK\tCarry on current calls.");
-
-	SNMP_TRAP(3, SNMPInfo, General, "Full config reloaded");
-
-	/*
-	** Leave critical Section
-	*/
-	// give other threads the chance to pass by this handler
-	PThread::Sleep(500);
+		// give other threads the chance to pass by this handler
+		PThread::Sleep(500);
+		/*
+		** Leave critical Section
+		*/
+		ReloadMutex.Signal();
+	}
 }
 
 #ifdef _WIN32
@@ -926,20 +951,20 @@ BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 {
 	PString eventName = "CTRL_UNKNOWN_EVENT";
 
-	if( dwCtrlType == CTRL_LOGOFF_EVENT ) {
+	if (dwCtrlType == CTRL_LOGOFF_EVENT) {
 		eventName = "CTRL_LOGOFF_EVENT";
 		PTRACE(2, "GK\tGatekeeper received " << eventName);
 		// prevent shut down
 		return FALSE;
 	}
 
-	if( dwCtrlType == CTRL_C_EVENT )
+	if (dwCtrlType == CTRL_C_EVENT)
 		eventName = "CTRL_C_EVENT";
-	else if( dwCtrlType == CTRL_BREAK_EVENT )
+	else if (dwCtrlType == CTRL_BREAK_EVENT)
 		eventName = "CTRL_BREAK_EVENT";
-	else if( dwCtrlType == CTRL_CLOSE_EVENT )
+	else if (dwCtrlType == CTRL_CLOSE_EVENT)
 		eventName = "CTRL_CLOSE_EVENT";
-	else if( dwCtrlType == CTRL_SHUTDOWN_EVENT )
+	else if (dwCtrlType == CTRL_SHUTDOWN_EVENT)
 		eventName = "CTRL_SHUTDOWN_EVENT";
 
 #ifndef hasPTLibTraceOnShutdownBug
@@ -959,7 +984,7 @@ BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 	// only safe way to handle this, is to remove the
 	// 'Close' menu item from the System menu and we will
 	// never have to deal with this event again.
-	if( dwCtrlType == CTRL_CLOSE_EVENT )
+	if (dwCtrlType == CTRL_CLOSE_EVENT)
 		WaitForSingleObject(GetCurrentProcess(), 15000);
 
 	// proceed with shut down
@@ -1033,19 +1058,18 @@ bool Gatekeeper::SetUserAndGroup(const PString & username)
 
 void UnixShutdownHandler(int sig)
 {
-	if (ShutdownMutex.WillBlock() || !RasServer::Instance()->IsRunning())
-		return;
-	PWaitAndSignal shutdown(ShutdownMutex);
-	PTRACE(1, "GK\tReceived signal " << sig);
+	if (RasServer::Instance()->IsRunning() && ShutdownMutex.Wait(0)) {
+		PTRACE(1, "GK\tReceived signal " << sig);
 #ifdef HAS_H46017
-	// unregister all H.460.17 endpoints before we stop the socket handlers and thus delete their sockets
-	RegistrationTable::Instance()->UnregisterAllH46017Endpoints();
+		// unregister all H.460.17 endpoints before we stop the socket handlers and thus delete their sockets
+		RegistrationTable::Instance()->UnregisterAllH46017Endpoints();
 #endif
-	PFile::Remove(pidfile);
-	RasServer::Instance()->Stop();
+		PFile::Remove(pidfile);
+		RasServer::Instance()->Stop();
+	}
 }
 
-void UnixReloadHandler(int sig) // For HUP Signal
+void UnixReloadHandler(int sig) // for HUP Signal
 {
 	PTRACE(1, "GK\tGatekeeper Hangup (signal " << sig << ")");
 	ReloadHandler();
@@ -1066,6 +1090,7 @@ Gatekeeper::Gatekeeper(const char * _manuf,
 	: PProcess(_manuf, _name, _majorVersion, _minorVersion, _status, _buildNumber)
 #endif
 {
+	m_strictConfigCheck = false;
 #ifdef _WIN32
 	// set data execution prevention (ignore if not available)
 	SetDEP(PROCESS_DEP_ENABLE);
@@ -1157,7 +1182,9 @@ const PString Gatekeeper::GetArgumentsParseString() const
 		 "-pid:"
 #ifdef P_LINUX
 		 "-core:"
+		 "-mlock."
 #endif
+		 "-strict."
 		 "h-help:"
 		 );
 }
@@ -1212,7 +1239,7 @@ bool Gatekeeper::InitHandlers(const PArgList & args)
 bool Gatekeeper::InitLogging(const PArgList & args)
 {
 	// Syslog is the default when compiled as service, but we don't want that
-	PTrace::ClearOptions(PTrace::SystemLogStream);
+	PTrace::ClearOptions(PTrace::SystemLogStream | PTrace::Thread | PTrace::Timestamp);
 	PTrace::SetOptions(PTrace::DateAndTime | PTrace::TraceLevel | PTrace::FileAndLine);
 	PTrace::SetLevel(args.GetOptionCount('t'));
 	if (args.HasOption('o')) {
@@ -1247,10 +1274,8 @@ bool Gatekeeper::InitConfig(const PArgList & args)
 
 	Toolkit::Instance()->SetConfig(fp, section);
 
-	// check config for unknown options (only warn about them)
-	CheckConfig(GkConfig(), section);
-
-	return true;
+	// check config for unknown options (only return failure when strict flag is set)
+	return (CheckConfig(GkConfig(), section) || !m_strictConfigCheck);
 }
 
 
@@ -1272,7 +1297,9 @@ void Gatekeeper::PrintOpts()
 		"      --pid file     : Specify the pid file\n"
 #ifdef P_LINUX
 		"      --core n       : Enable core dumps (with max size of n bytes)\n"
+		"      --mlock        : Lock GnuGk into memory to prevent it being swaped out\n"
 #endif
+		"      --strict       : Strict config check (don't start with errors in config)\n"
 		"  -h  --help         : Show this message\n" << endl;
 }
 
@@ -1325,13 +1352,34 @@ void Gatekeeper::Main()
 			}
 		}
 	}
+	// lock in memory
+	if (args.HasOption("mlock")) {
+	    struct rlimit rlim;
+	    rlim.rlim_max = RLIM_INFINITY;
+	    rlim.rlim_cur = RLIM_INFINITY;
+	    cout << "Trying to lock GnuGk into RAM" << endl;
+	    PTRACE(1, "Trying to lock GnuGk into RAM");
+	    if (setrlimit(RLIMIT_MEMLOCK, &rlim) < 0) {
+			cout << "setrlimit() failed: Not locking into RAM" << endl;
+		    PTRACE(1, "setrlimit() failed: Not locking into RAM");
+	    }
+	    else {
+			if (mlockall(MCL_CURRENT | MCL_FUTURE) < 0) {
+				cout << "Cannot lock GnuGk into RAM, mlockall() failed" << endl;
+			    PTRACE(1, "Cannot lock GnuGk into RAM, mlockall() failed");
+			} else {
+				cout << "GnuGk successfully locked into RAM" << endl;
+			    PTRACE(1, "GnuGk successfully locked into RAM");
+			}
+	    }
+	}
 #endif
 
 #ifdef HAS_SETUSERNAME
 	if (args.HasOption('u')) {
 		const PString username = args.GetOptionString('u');
 
-		if ( !SetUserAndGroup(username) ) {
+		if (!SetUserAndGroup(username)) {
 			cout << "GNU Gatekeeper could not run as user "
 				<< username
 				<< endl;
@@ -1340,7 +1388,7 @@ void Gatekeeper::Main()
 	}
 #endif
 
-	if(!InitLogging(args))
+	if (!InitLogging(args))
 		return;
 
 	if (args.HasOption('h')) {
@@ -1348,6 +1396,7 @@ void Gatekeeper::Main()
 		ExitGK();
 	}
 
+	m_strictConfigCheck = args.HasOption("strict");
 	if (!InitConfig(args) || !InitHandlers(args))
 		ExitGK();
 
@@ -1493,8 +1542,8 @@ void Gatekeeper::Main()
 	FreeConsole();
 #endif // _WIN32
 
-#ifdef P_OPENBSD
-	_exit(0);	// skip exit handlers, will hang on OpenBSD 5.3
+#if defined(P_OPENBSD) || defined(P_MACOSX)
+	_exit(0);	// skip exit handlers, will hang on OpenBSD 5.3 and crash on MacOSX
 #endif
 }
 
