@@ -13,7 +13,7 @@
 
 
 #ifndef GK_CONST_H
-#define GK_CONST_H "@(#) $Id: gk_const.h,v 1.32 2013/08/02 22:12:08 willamowius Exp $"
+#define GK_CONST_H "@(#) $Id: gk_const.h,v 1.35 2014/01/02 23:35:16 willamowius Exp $"
 
 #include "config.h"
 
@@ -22,11 +22,11 @@
  */
 
 #define GK_DEF_UNICAST_RAS_PORT		1719
-#define GK_DEF_MULTICAST_PORT			1718
+#define GK_DEF_MULTICAST_PORT		1718
 #define GK_DEF_MULTICAST_GROUP		"224.0.1.41"
 
-/* port used by gatekeeper for routed signaling: anything != 1720 so endpoint can be on same IP as GK */
-#define GK_DEF_CALL_SIGNAL_PORT		1721
+/* port used by gatekeeper for routed signaling */
+#define GK_DEF_CALL_SIGNAL_PORT		1720
 #define GK_DEF_TLS_CALL_SIGNAL_PORT	1300
 
 /* well known signal port */
@@ -34,11 +34,13 @@
 
 #define GK_DEF_STATUS_PORT			7000
 
-#define GK_DEF_MULTIPLEX_RTP_PORT		3000
+#define GK_DEF_MULTIPLEX_RTP_PORT	3000
 #define GK_DEF_MULTIPLEX_RTCP_PORT	3001
 
 #define GK_DEF_LISTEN_QUEUE_LENGTH	1024
 
+#define GK_DEF_BANDWIDTH			1280
+
 extern const char *H225_ProtocolID;
 extern const char *H245_ProtocolID;
 
@@ -58,6 +60,14 @@ static const char * const H46018_OID = "0.0.8.460.18.0.1";
 static const char * const H46019_OID = "0.0.8.460.19.0.1";
 #endif
 
+
+///////////////////////////////////////////
+// H.460.22
+#define Std22_TLS				1	// detect TLS support
+#define Std22_IPSec				2	// detect IPSec support
+#define Std22_Priority			1	// priority
+#define Std22_ConnectionAddress	2	// connectionAddress
+
 ///////////////////////////////////////////
 // H.460.23
 
