@@ -43,6 +43,208 @@ using std::for_each;
 using std::find_if;
 using std::greater;
 
+
+#if defined(H323_H235) && hasCipertextStealing
+////////////////////////////////////////////////////
+
+#include <h235/h235crypto.h>
+
+/** This class implements desECB authentication.
+*/
+class H235AuthDesECB : public H235Authenticator
+{
+    PCLASSINFO(H235AuthDesECB, H235Authenticator);
+  public:
+    H235AuthDesECB();
+
+    PObject * Clone() const;
+
+    virtual const char * GetName() const;
+
+    static PStringArray GetAuthenticatorNames();
+#if PTLIB_VER >= 2110
+    static PBoolean GetAuthenticationCapabilities(Capabilities * ids);
+#endif
+    virtual PBoolean IsMatch(const PString & identifier) const;
+
+    virtual H225_CryptoH323Token * CreateCryptoToken();
+
+    virtual ValidationResult ValidateCryptoToken(
+      const H225_CryptoH323Token & cryptoToken,
+      const PBYTEArray & rawPDU
+    );
+
+    virtual PBoolean IsCapability(
+      const H235_AuthenticationMechanism & mechansim,
+      const PASN_ObjectId & algorithmOID
+    );
+
+    virtual PBoolean SetCapability(
+      H225_ArrayOf_AuthenticationMechanism & mechansim,
+      H225_ArrayOf_PASN_ObjectId & algorithmOIDs
+    );
+
+    virtual PBoolean IsSecuredPDU(
+      unsigned rasPDU,
+      PBoolean received
+    ) const;
+
+    virtual PBoolean IsSecuredSignalPDU(
+      unsigned rasPDU,
+      PBoolean received
+    ) const;
+};
+
+#if PTLIB_VER >= 2110
+// disabled on PTLib 2.11.x, crashes on startup
+#if PTLIB_VER >= 2120
+typedef H235AuthDesECB H235AuthenticatorDesECB;
+PPLUGIN_STATIC_LOAD(DesECB,H235Authenticator);
+H235SECURITY(DesECB);
+#endif
+#else
+static PFactory<H235Authenticator>::Worker<H235AuthDesECB> factoryH235AuthDesECB("desECB");
+#endif
+
+static const char OID_DesECB[] = "1.3.14.3.2.6";
+
+H235AuthDesECB::H235AuthDesECB()
+{
+  usage = AnyApplication; // Can be used either for GKAdmission or EPAuthenticstion
+}
+
+PObject * H235AuthDesECB::Clone() const
+{
+  return new H235AuthDesECB(*this);
+}
+
+const char * H235AuthDesECB::GetName() const
+{
+  return "desECB";
+}
+
+PStringArray H235AuthDesECB::GetAuthenticatorNames()
+{
+  return PStringArray("desECB");
+}
+
+#if PTLIB_VER >= 2110
+PBoolean H235AuthDesECB::GetAuthenticationCapabilities(H235Authenticator::Capabilities * ids)
+{
+  H235Authenticator::Capability cap;
+  cap.m_identifier = OID_DesECB;
+  cap.m_cipher     = "DES";
+  cap.m_description= "desECB";
+  ids->capabilityList.push_back(cap);
+    
+  return true;
+}
+#endif
+
+PBoolean H235AuthDesECB::IsMatch(const PString & identifier) const 
+{ 
+    return (identifier == PString(OID_DesECB)); 
+}
+
+H225_CryptoH323Token * H235AuthDesECB::CreateCryptoToken()
+{
+  if (!IsActive())
+    return NULL;
+
+  // Create the H.225 crypto token
+  H225_CryptoH323Token * cryptoToken = new H225_CryptoH323Token;
+  cryptoToken->SetTag(H225_CryptoH323Token::e_cryptoEPPwdEncr);
+
+  // TODO: encryption not implemented, yet (GnuGk only needs the decrypt)
+
+  return cryptoToken;
+}
+
+H235Authenticator::ValidationResult H235AuthDesECB::ValidateCryptoToken(
+                                             const H225_CryptoH323Token & cryptoToken,
+                                             const PBYTEArray &)
+{
+  if (!IsActive())
+	return e_Disabled;
+
+  // verify the token is of correct type
+  if (cryptoToken.GetTag() != H225_CryptoH323Token::e_cryptoEPPwdEncr)
+    return e_Absent;
+
+  PBYTEArray remoteEncryptedData = ((H235_ENCRYPTED<H235_EncodedPwdCertToken>)cryptoToken).m_encryptedData;
+  PBYTEArray decryptedToken(remoteEncryptedData.GetSize());
+
+  EVP_CIPHER_CTX cipher;
+  EVP_CIPHER_CTX_init(&cipher);
+  EVP_CIPHER_CTX_set_padding(&cipher, 1);
+
+  PBYTEArray key(8);
+  // Build key from password according to H.235.0/8.2.1
+  memcpy(key.GetPointer(), (const char *)password, std::min(key.GetSize(), password.GetLength()));
+  for (PINDEX i = key.GetSize(); i < password.GetLength(); ++i)
+	key[i%key.GetSize()] ^= password[i];
+
+  EVP_CipherInit_ex(&cipher, EVP_des_ecb(), NULL, key, NULL, 0);
+
+  int len = -1;
+  if (!EVP_DecryptUpdate_cts(&cipher, decryptedToken.GetPointer(), &len, remoteEncryptedData.GetPointer(), remoteEncryptedData.GetSize())) {
+        PTRACE(1, "H235RAS\tEVP_DecryptUpdate_cts failed");
+  }
+  int f_len = -1;
+  if(!EVP_DecryptFinal_cts(&cipher, decryptedToken.GetPointer() + len, &f_len)) {
+    char buf[256];
+    ERR_error_string(ERR_get_error(), buf);
+    PTRACE(1, "H235RAS\tEVP_DecryptFinal_cts failed: " << buf);
+  }
+
+  EVP_CIPHER_CTX_cleanup(&cipher);
+
+  PPER_Stream asn(decryptedToken);
+  H235_ClearToken clearToken;
+  clearToken.Decode(asn);
+
+  PString generalID = clearToken.m_generalID;
+  if (generalID == Toolkit::GKName()
+	  && clearToken.m_timeStamp == (unsigned)time(NULL))	// TODO: add grace period ?
+	return e_OK;
+
+  PTRACE(1, "H235RAS\tH235AuthDesECB password does not match.");
+  return e_BadPassword;
+}
+
+PBoolean H235AuthDesECB::IsCapability(const H235_AuthenticationMechanism & mechanism,
+                                     const PASN_ObjectId & algorithmOID)
+{
+  return mechanism.GetTag() == H235_AuthenticationMechanism::e_pwdSymEnc &&
+         algorithmOID.AsString() == OID_DesECB;
+}
+
+PBoolean H235AuthDesECB::SetCapability(H225_ArrayOf_AuthenticationMechanism & mechanisms,
+                                      H225_ArrayOf_PASN_ObjectId & algorithmOIDs)
+{
+  return AddCapability(H235_AuthenticationMechanism::e_pwdSymEnc, OID_DesECB, mechanisms, algorithmOIDs);
+}
+
+PBoolean H235AuthDesECB::IsSecuredPDU(unsigned rasPDU, PBoolean received) const
+{
+  switch (rasPDU) {
+    case H225_RasMessage::e_registrationRequest :
+      return received ? !remoteId.IsEmpty() : !localId.IsEmpty();
+
+    default :
+      return FALSE;
+  }
+}
+
+PBoolean H235AuthDesECB::IsSecuredSignalPDU(unsigned signalPDU, PBoolean received) const
+{
+  return FALSE;
+}
+
+/////////////////////////////////////////////////////////////////////////////
+#endif
+
+
 ARQAuthData::ARQAuthData(
 	/// an endpoint requesting admission
 	const endptr & ep,
@@ -86,7 +288,7 @@ void ARQAuthData::SetRouteToAlias(const H225_ArrayOf_AliasAddress & alias)
 	m_routeToAlias = alias;
 }
 
-void ARQAuthData::SetRouteToAlias(const PString& alias, int tag)
+void ARQAuthData::SetRouteToAlias(const PString & alias, int tag)
 {
 	m_routeToAlias.SetSize(1);
 	H323SetAliasAddress(alias, m_routeToAlias[0], tag);
@@ -94,7 +296,7 @@ void ARQAuthData::SetRouteToAlias(const PString& alias, int tag)
 
 SetupAuthData::SetupAuthData(
 	/// call associated with the message (if any)
-	const callptr& call,
+	const callptr & call,
 	/// is the Setup message from a registered endpoint
 	bool fromRegistered,
 	/// did the Setup come in over TLS
@@ -115,7 +317,7 @@ SetupAuthData::SetupAuthData(const SetupAuthData & obj)
 {
 }
 
-SetupAuthData& SetupAuthData::operator=(const SetupAuthData& obj)
+SetupAuthData& SetupAuthData::operator=(const SetupAuthData & obj)
 {
 	if (this != &obj) {
 		m_rejectReason = obj.m_rejectReason;
@@ -142,7 +344,7 @@ void SetupAuthData::SetRouteToAlias(const H225_ArrayOf_AliasAddress & alias)
 	m_routeToAlias = alias;
 }
 
-void SetupAuthData::SetRouteToAlias(const PString& alias, int tag)
+void SetupAuthData::SetRouteToAlias(const PString & alias, int tag)
 {
 	m_routeToAlias.SetSize(1);
 	H323SetAliasAddress(alias, m_routeToAlias[0], tag);
@@ -160,9 +362,7 @@ GkAuthenticator::GkAuthenticator(
 	m_enabledMiscChecks(~0U), m_supportedMiscChecks(supportedMiscChecks),
 	m_config(GkConfig())
 {
-	const PStringArray control(
-		m_config->GetString(GkAuthSectionName, name, "").Tokenise(";,")
-		);
+	const PStringArray control(m_config->GetString(GkAuthSectionName, name, "").Tokenise(";,"));
 	if (control.GetSize() > 0) {
 		const PString controlStr = control[0].Trim();
 		if (strcasecmp(name, "default") == 0)
@@ -332,23 +532,6 @@ int GkAuthenticator::Check(
 		? m_defaultStatus : e_next;
 }
 
-// TODO: unused ? remove ?
-bool GkAuthenticator::GetH235Capability(
-	/// append supported authentication mechanism to this array
-	H225_ArrayOf_AuthenticationMechanism & mechanisms,
-	/// append supported algorithm OIDs for the given authentication
-	/// mechanism
-	H225_ArrayOf_PASN_ObjectId & algorithmOIDs
-	) const
-{
-	if (m_h235Authenticators && m_h235Authenticators->GetSize() > 0) {
-		for (PINDEX i = 0; i < m_h235Authenticators->GetSize(); i++)
-			(*m_h235Authenticators)[i].SetCapability(mechanisms, algorithmOIDs);
-		return true;
-	} else
-		return false;
-}		
-
 bool GkAuthenticator::IsH235Capability(
 	/// authentication mechanism
 	const H235_AuthenticationMechanism & mechanism,
@@ -384,7 +567,7 @@ void GkAuthenticator::AppendH235Authenticator(
 
 PString GkAuthenticator::GetUsername(
 	/// RRQ message with additional data
-	const RasPDU<H225_RegistrationRequest>& request
+	const RasPDU<H225_RegistrationRequest> & request
 	) const
 {
 	const H225_RegistrationRequest& rrq = request;
@@ -689,18 +872,19 @@ void GkAuthenticatorList::SelectH235Capability(
 	H225_GatekeeperConfirm & gcf)
 {
 	ReadLock lock(m_reloadMutex);
-	
-	if (m_authenticators.empty())
+
+	if (m_authenticators.empty()) {
 		return;
+	}
 
 	// if GRQ does not contain a list of authentication mechanisms simply return
 	if (!(grq.HasOptionalField(H225_GatekeeperRequest::e_authenticationCapability)
 			&& grq.HasOptionalField(H225_GatekeeperRequest::e_algorithmOIDs)
 			&& grq.m_authenticationCapability.GetSize() > 0
-			&& grq.m_algorithmOIDs.GetSize() > 0))
+			&& grq.m_algorithmOIDs.GetSize() > 0)) {
 		return;
+	}
 
-	// TODO: add TLS to negotiation either here or in H323Plus ?
 	for (PINDEX auth = 0; auth < m_h235authenticators.GetSize(); auth++) {
 		for (PINDEX cap = 0; cap < grq.m_authenticationCapability.GetSize(); cap++) {
 			for (PINDEX alg = 0; alg < grq.m_algorithmOIDs.GetSize(); alg++) {
@@ -721,7 +905,7 @@ void GkAuthenticatorList::SelectH235Capability(
 								gcf.m_tokens.SetSize(1);
 								gcf.m_tokens[0].m_tokenOID = "0.0";
 								gcf.m_tokens[0].IncludeOptionalField(H235_ClearToken::e_timeStamp);
-								gcf.m_tokens[0].m_timeStamp = (int)time(NULL);
+								gcf.m_tokens[0].m_timeStamp = (int)time(NULL); // Avaya seems to send a different timestamp that is 34 years back, but accpets this as well
 								gcf.m_tokens[0].IncludeOptionalField(H235_ClearToken::e_random);
 								gcf.m_tokens[0].m_random = rand();
 								gcf.m_tokens[0].IncludeOptionalField(H235_ClearToken::e_generalID);
@@ -738,9 +922,9 @@ void GkAuthenticatorList::SelectH235Capability(
 
 bool GkAuthenticatorList::Validate(
 	/// RRQ to be validated by authenticators
-	RasPDU<H225_RegistrationRequest>& request,
+	RasPDU<H225_RegistrationRequest> & request,
 	/// authorization data (reject reason, ...)
-	RRQAuthData& authData
+	RRQAuthData & authData
 	)
 {
 	ReadLock lock(m_reloadMutex);
@@ -849,7 +1033,7 @@ bool GkAuthenticatorList::Validate(
 // class CacheManager
 bool CacheManager::Retrieve(
 	const PString & key, /// the key to look for
-	PString&  value /// filled with the value on return
+	PString & value /// filled with the value on return
 	) const
 {
 	// quick check
@@ -944,8 +1128,7 @@ SimplePasswordAuth::~SimplePasswordAuth()
 
 int SimplePasswordAuth::Check(
 	RasPDU<H225_RegistrationRequest> & request, 
-	RRQAuthData& authData
-	)
+	RRQAuthData & authData)
 {
 	H225_RegistrationRequest& rrq = request;
 	return doCheck(request, 
@@ -1025,184 +1208,6 @@ bool SimplePasswordAuth::InternalGetPassword(
 		return false;
 }
 
-int SimplePasswordAuth::CheckTokens(
-	/// an array of tokens to be checked
-	const H225_ArrayOf_ClearToken & tokens,
-	/// aliases for the endpoint that generated the tokens
-	const H225_ArrayOf_AliasAddress * aliases
-	)
-{
-	for (PINDEX i = 0; i < tokens.GetSize(); i++) {
-		H235_ClearToken& token = tokens[i];
-
-		// check for Cisco Access Token
-		if (token.m_tokenOID == OID_CAT) {
-			if (!token.HasOptionalField(H235_ClearToken::e_generalID)) {
-				PTRACE(3, "GKAUTH\t" << GetName() << " generalID field "
-					"not found inside CAT token");
-				return e_fail;
-			}
-			const PString id = token.m_generalID;
-			if (m_checkID && (aliases == NULL || FindAlias(*aliases, id) == P_MAX_INDEX)) {
-				PTRACE(3, "GKAUTH\t" << GetName() << " generalID '" << id
-					<< "' of CAT token does not match any alias for the endpoint");
-				return e_fail;
-			}
-			
-			PString passwd;
-			if (!InternalGetPassword(id, passwd)) {
-				PTRACE(3, "GKAUTH\t" << GetName() << " password not found for '" << id << '\'');
-				return e_fail;
-			}
-
-			H235AuthCAT authCAT;
-			authCAT.SetLocalId(id);
-			authCAT.SetPassword(passwd);
-			if (authCAT.ValidateClearToken(token) == H235Authenticator::e_OK) {
-				PTRACE(5, "GKAUTH\t" << GetName() << " CAT password match for '" << id << '\'');
-				return e_ok;
-			} else
-				return e_fail;
-		}
-		
-		if (token.HasOptionalField(H235_ClearToken::e_password)) {
-			if (!token.HasOptionalField(H235_ClearToken::e_generalID)) {
-				PTRACE(3, "GKAUTH\t"<< GetName() << " generalID field not found"
-					<<" inside the clear text token");
-				return e_fail;
-			}
-			const PString id = token.m_generalID;
-			if (m_checkID && (aliases == NULL || FindAlias(*aliases, id) == P_MAX_INDEX)) {
-				PTRACE(3, "GKAUTH\t" << GetName() << " generalID '"
-					<< "' does not match any alias for the endpoint");
-				return e_fail;
-			}
-
-			PString passwd;
-			if (!InternalGetPassword(id, passwd)) {
-				PTRACE(3, "GKAUTH\t" << GetName() << " password not found for '" << id << '\'');
-				return e_fail;
-			}
-			
-			const PString tokenpasswd = token.m_password;
-			if (passwd == tokenpasswd) {
-				PTRACE(5, "GKAUTH\t" << GetName() << " clear text password "
-					"match for '" << id << '\'');
-				return e_ok;
-			} else {
-				return e_fail;
-			}
-		}
-	}
-	return e_next;
-}
-
-// TODO: unused ? remove ?
-int SimplePasswordAuth::CheckCryptoTokens(
-	/// an array of cryptoTokens to be checked
-	const H225_ArrayOf_CryptoH323Token & tokens, 
-	/// aliases for the endpoint that generated the tokens
-	const H225_ArrayOf_AliasAddress * aliases,
-	/// raw data for RAS PDU - required to validate some tokens
-	/// like H.235 Auth Procedure I
-	const PBYTEArray & rawPDU)
-{
-	for (PINDEX i = 0; i < tokens.GetSize(); i++) {
-		if (tokens[i].GetTag() == H225_CryptoH323Token::e_cryptoEPPwdHash) {
-			H225_CryptoH323Token_cryptoEPPwdHash& pwdhash = tokens[i];
-			const PString id = AsString(pwdhash.m_alias, false);
-			if (m_checkID && (aliases == NULL || FindAlias(*aliases, id) == P_MAX_INDEX)) {
-				PTRACE(3, "GKAUTH\t" << GetName() << " alias '" << id 
-					<< "' of the cryptoEPPwdHash token does not match "
-					"any alias for the endpoint");
-				return e_fail;
-			}
-
-			PString passwd;
-			if (!InternalGetPassword(id, passwd)) {
-				PTRACE(3, "GKAUTH\t" << GetName() << " password not found for '" << id << '\'');
-				return e_fail;
-			}
-			
-			H235AuthSimpleMD5 authMD5;
-			authMD5.SetLocalId(id);
-			authMD5.SetPassword(passwd);
-			if (authMD5.ValidateCryptoToken(tokens[i], rawPDU) == H235Authenticator::e_OK) {
-				PTRACE(5, "GKAUTH\t" << GetName() << " MD5 password match for '" << id << '\'');
-				return e_ok;
-			} else
-				return e_fail;
-#if P_SSL
-		} else if (tokens[i].GetTag() == H225_CryptoH323Token::e_nestedcryptoToken) {
-			const H235_CryptoToken& nestedCryptoToken = tokens[i];
-			
-			if (nestedCryptoToken.GetTag() != H235_CryptoToken::e_cryptoHashedToken)
-				continue;
-				
-			const H235_CryptoToken_cryptoHashedToken& cryptoHashedToken = nestedCryptoToken;
-			const H235_ClearToken& clearToken = cryptoHashedToken.m_hashedVals;
-			
-			if (!clearToken.HasOptionalField(H235_ClearToken::e_sendersID)) {
-				PTRACE(5, "GKAUTH\t" << GetName() << " hashedVals of nested "
-					" cryptoHashedToken do not contain sendersID");
-				continue;
-			}
-			
-			PString id = clearToken.m_sendersID; 
-			if (m_checkID && (aliases == NULL || FindAlias(*aliases, id) == P_MAX_INDEX)) {
-				PTRACE(3, "GKAUTH\t" << GetName() << " sendersID '" << id 
-					<< "' of the cryptoHashedToken hasgedVals does not match "
-					"any alias for the endpoint");
-				return e_fail;
-			}
-			
-			PString passwd;
-			bool passwordFound = InternalGetPassword(id, passwd);
-			
-			//if a password is not found: senderID == endpointIdentifier?
-			if (!passwordFound) {
-				H225_EndpointIdentifier epId;
-				epId = id;
-				endptr ep = RegistrationTable::Instance()->FindByEndpointId(epId);
-				if (!ep) {
-					PTRACE(3, "GKAUTH\t" << GetName() << " sendersID '" << id 
-						<< "' of the cryptoHashedToken hashedVals does not match "
-						"any endpoint identifier");
-					return e_fail;
-				}
-				
-				// check all endpoint aliases for a password
-				const H225_ArrayOf_AliasAddress pwaliases = ep->GetAliases();
-				for (PINDEX p = 0; p < pwaliases.GetSize(); p++) {
-					id = AsString(pwaliases[p], FALSE);
-					passwordFound = InternalGetPassword(id, passwd);
-					if (passwordFound)
-						break;
-				}
-			}
-			
-			if (!passwordFound) {
-				PTRACE(3, "GKAUTH\t" << GetName() << " password not found for '" << id << '\'');
-				return e_fail;
-			}
-			
-			H235AuthProcedure1 authProcedure1;
-			authProcedure1.SetLocalId(Toolkit::GKName());
-			authProcedure1.SetPassword(passwd);
-			const int result = authProcedure1.ValidateCryptoToken(tokens[i], rawPDU);
-			if (result == H235Authenticator::e_OK) {
-				PTRACE(5, "GKAUTH\t" << GetName() << " SHA-1 password match for '" << id << '\'');
-				return e_ok;
-			} else if (result == H235Authenticator::e_Absent)
-				continue;
-			else
-				return e_fail;
-#endif
-		}
-	}
-	return e_next;
-}
-
 bool SimplePasswordAuth::ResolveUserName(const H235_ClearToken & token, PString & username)
 {
 	// CAT
