@@ -144,6 +144,29 @@ PString AsDotString(const H225_TransportAddress & addr, bool showPort)
 	return result;
 }
 
+void SetH225Port(H225_TransportAddress & addr, WORD port)
+{
+	if (addr.GetTag() == H225_TransportAddress::e_ip6Address) {
+		H225_TransportAddress_ip6Address & ipv6 = addr;
+		ipv6.m_port = port;
+	} else if (addr.GetTag() == H225_TransportAddress::e_ipAddress) {
+		H225_TransportAddress_ipAddress & ipv4 = addr;
+		ipv4.m_port = port;
+	}
+}
+
+WORD GetH225Port(const H225_TransportAddress & addr)
+{
+	if (addr.GetTag() == H225_TransportAddress::e_ip6Address) {
+		const H225_TransportAddress_ip6Address & ipv6 = addr;
+		return ipv6.m_port;
+	} else if (addr.GetTag() == H225_TransportAddress::e_ipAddress) {
+		const H225_TransportAddress_ipAddress & ipv4 = addr;
+		return ipv4.m_port;
+	}
+	return 0;
+}
+
 PString AsString(const H323TransportAddress & ta)
 {
 	if (!IsSet(ta))
@@ -354,6 +377,15 @@ H225_TransportAddress SocketToH225TransportAddr(const PIPSocket::Address & Addr,
 	return Result;
 }
 
+// convert a H.323 transport address into an H225 transport address
+H225_TransportAddress H323ToH225TransportAddress(const H323TransportAddress & h323addr)
+{
+	PIPSocket::Address ip;
+	WORD port = 0;
+	h323addr.GetIpAndPort(ip, port);
+	return SocketToH225TransportAddr(ip, port);
+}
+
 // convert a H.245 unicast address into an H.323 transport address
 H323TransportAddress H245UnicastToH323TransportAddr(const H245_UnicastAddress & h245unicast)
 {
