@@ -13,7 +13,7 @@
 
 
 #ifndef H323UTIL_H
-#define H323UTIL_H "@(#) $Id: h323util.h,v 1.54 2013/09/12 10:51:58 willamowius Exp $"
+#define H323UTIL_H "@(#) $Id: h323util.h,v 1.56 2013/10/24 12:32:58 willamowius Exp $"
 
 #include <ptlib.h>
 #include <ptlib/sockets.h>
@@ -74,6 +74,9 @@ H245_TransportAddress IPToH245TransportAddr(const PIPSocket::Address & ip, WORD
 // convert a socket IP address into an H225 transport address
 H225_TransportAddress SocketToH225TransportAddr(const PIPSocket::Address & Addr, WORD Port);
 
+// convert a H.323 transport address into an H225 transport address
+H225_TransportAddress H323ToH225TransportAddress(const H323TransportAddress & h323addr);
+
 // convert a H.245 unicast address into an H.323 transport address
 H323TransportAddress H245UnicastToH323TransportAddr(const H245_UnicastAddress & h245unicast);
 
@@ -87,6 +90,9 @@ bool GetIPAndPortFromTransportAddr(const H225_TransportAddress & addr, PIPSocket
 
 PStringArray SplitIPAndPort(const PString & str, WORD default_port);
 
+WORD GetH225Port(const H225_TransportAddress & addr);
+void SetH225Port(H225_TransportAddress & addr, WORD port);
+
 WORD GetH245Port(const H245_UnicastAddress & addr);
 void SetH245Port(H245_UnicastAddress & addr, WORD port);
 
