@@ -22,8 +22,11 @@
 #include "gk_const.h"
 #include "snmp.h"
 
-#include <lua.hpp>
-
+extern "C" {
+#include "lua.h"
+#include "lualib.h"
+#include "lauxlib.h"
+}
 
 namespace Routing {
 
@@ -46,6 +49,7 @@ protected:
 		const PString & callid,
 		const PString & messageType,
 		const PString & clientauthid,
+		const PString & language,
 		/* out: */
 		DestinationRoutes & destination);
 
@@ -141,6 +145,7 @@ void LuaPolicy::RunPolicy(
 		const PString & callid,
 		const PString & messageType,
 		const PString & clientauthid,
+		const PString & language,
 		/* out: */
 		DestinationRoutes & destination)
 {
@@ -152,6 +157,7 @@ void LuaPolicy::RunPolicy(
 	SetValue("callid", callid);
 	SetValue("messageType", messageType);
 	SetValue("clientauthid", clientauthid);
+	SetValue("language", language);
 
 	if (luaL_loadstring(m_lua, m_script) != 0 || lua_pcall(m_lua, 0, 0, 0) != 0) {
 		PTRACE(1, "LUA\tError in LUA script: " << lua_tostring(m_lua, -1));
@@ -189,7 +195,7 @@ void LuaPolicy::RunPolicy(
 	if (!destIP.IsEmpty()) {
 		PTRACE(5, m_name << "\tSet new destination IP " << destIP);
 		PStringArray adr_parts = SplitIPAndPort(destIP, GK_DEF_ENDPOINT_SIGNAL_PORT);
-		PString ip = adr_parts[0];
+		PIPSocket::Address ip(adr_parts[0]);
 		WORD port = (WORD)(adr_parts[1].AsInteger());
 
 		Route route("Lua", SocketToH225TransportAddr(ip, port));
