@@ -16,9 +16,8 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef NAME_H
-#define NAME_H "@(#) $Id: name.h,v 1.10 2010/10/25 15:01:51 willamowius Exp $"
+#define NAME_H "@(#) $Id: name.h,v 1.11 2011/11/28 12:33:41 willamowius Exp $"
 
-#if PTRACING
 
 class NamedObject 
 {
@@ -67,16 +66,4 @@ private:
 	PString m_name;
 };
 
-#else
-
-struct NamedObject {
-	NamedObject(
-		/// name to set for the object
-		const char* name = NULL
-		) {}
-	void SetName(const char*) {}
-};
-
-#endif
-
 #endif // NAME_H
