@@ -16,7 +16,7 @@
 #if HAS_RADIUS
 
 #ifndef __RADACCT_H
-#define __RADACCT_H "@(#) $Id: radacct.h,v 1.16 2011/09/23 09:28:52 willamowius Exp $"
+#define __RADACCT_H "@(#) $Id: radacct.h,v 1.17 2011/12/07 22:42:32 willamowius Exp $"
 
 #include "RasTbl.h"
 #include "gkacct.h"
@@ -57,31 +57,6 @@ public:
 		const callptr& call
 		);
 		
-protected:
-	/** Called before Accounting-Request PDU is send.
-		Can be used to introduce additional attributes etc.
-		
-		@return
-		True to proceed, false to fail and not send this pdu.
-	*/
-	virtual bool OnSendPDU(
-		RadiusPDU& pdu, /// PDU to be sent
-		AcctEvent evt, /// accounting event being processed
-		const callptr& call /// call associated with this request (if any)
-		);
-
-	/** Called after Accounting-Response PDU is received.
-		Can be used to check for some additional attributes etc.
-		
-		@return
-		True to accept the response, false to return failure for this event.
-	*/
-	virtual bool OnReceivedPDU(
-		RadiusPDU& pdu, /// PDU received from RADIUS server
-		AcctEvent evt, /// accounting event being processed
-		const callptr& call /// call associated with this response (if any)
-		);
-		
 private:
 	RadAcct();
 	/* No copy constructor allowed */
