@@ -4,7 +4,7 @@
  * Structures to hold and process signaling messages
  *
  * Copyright (c) 2005, Michal Zygmuntowicz
- * Copyright (c) 2005-2010, Jan Willamowius
+ * Copyright (c) 2005-2013, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -14,7 +14,7 @@
  */
 
 #ifndef SIGMSG_H
-#define SIGMSG_H "@(#) $Id: sigmsg.h,v 1.10 2010/10/25 15:01:51 willamowius Exp $"
+#define SIGMSG_H "@(#) $Id: sigmsg.h,v 1.11 2013/09/26 20:25:10 willamowius Exp $"
 
 
 #include <ptlib/sockets.h>
@@ -179,7 +179,7 @@ typedef H225SignalingMsg<H225_ReleaseComplete_UUIE> ReleaseCompleteMsg;
 typedef H225SignalingMsg<H225_Information_UUIE> InformationMsg;
 typedef H225SignalingMsg<H225_Facility_UUIE> FacilityMsg;
 //typedef H225SignalingMsg<H225_Notify_UUIE> NotifyMsg;
-//typedef H225SignalingMsg<H225_Status_UUIE> StatusMsg;
+typedef H225SignalingMsg<H225_Status_UUIE> StatusMsg;
 //typedef H225SignalingMsg<H225_StatusInquiry_UUIE> StatusInquiryMsg;
 
 #endif // SIGMSG_H
