@@ -2,7 +2,7 @@
 //
 // snmp.cxx for GNU Gatekeeper
 //
-// Copyright (c) 2012, Jan Willamowius
+// Copyright (c) 2012-2014, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -221,7 +221,7 @@ int catchall_handler(netsnmp_mib_handler * /* handler */,
 	return SNMPERR_SUCCESS;
 }
 
-}
+} // extern "C"
 
 
 class NetSNMPAgent : public Singleton<NetSNMPAgent>
@@ -422,35 +422,8 @@ PBoolean PTLibSNMPAgent::OnGetNextRequest(PINDEX reqID, PSNMP::BindingList & var
 
 PBoolean PTLibSNMPAgent::OnSetRequest(PINDEX reqID, PSNMP::BindingList & vars, PSNMP::ErrorType & errCode)
 {
-	return PFalse;	// doesn't work
-
-/*
 	// TODO: SET operation is broken in PTLib PSNMPServer
-	// PSNMPServer::ProcessPDU() must send a response and the decoding of SET values is broken
-	for(PSNMP::BindingList::iterator i = vars.begin(); i != vars.end(); ++i){
-		if (i->first == TraceLevelOIDStr + PString(".0")) {
-			if (i->second.GetObject().GetTag() == PASN_Object::UniversalInteger) {
-				PASN_Integer & num = (PASN_Integer &)(i->second.GetObject());
-				PTRACE(0, "JW SET " << i->first << " to " << num << " = " << num.GetValue());
-				//PTrace::SetLevel(num.GetValue());
-				return PTrue;
-			} else {
-				PTRACE(1, "SNMP\tWrong data type for SET " << i->first);
-				return PFalse;
-			}
-		} else if (i->first == CatchAllOIDStr + PString(".0")) {
-			if (i->second.GetObject().GetTag() == PASN_Object::UniversalOctetString) {
-				PASN_OctetString & str = (PASN_OctetString &)(i->second.GetObject());
-				PTRACE(0, "JW SET " << i->first << " to " << str << " = " << str.GetValue());
-				return PTrue;
-			} else {
-				PTRACE(1, "SNMP\tWrong data type for SET " << i->first);
-				return PFalse;
-			}
-		}
-	}
-	return PFalse;
-*/
+	return PFalse;	// doesn't work
 }
 
 // 0=SNMPv1, 1=SNMPv2
@@ -492,7 +465,7 @@ PBoolean PTLibSNMPAgent::MIB_LocalMatch(PSNMP_PDU & answerPDU)
 	PSNMP_VarBindList & vars = answerPDU.m_variable_bindings;
 	bool found = false;
 
-	for(PINDEX i = 0; i < vars.GetSize(); i++){
+	for (PINDEX i = 0; i < vars.GetSize(); i++){
 		if (vars[i].m_name == ShortVersionOIDStr + PString(".0")) {
 			SetRFC1155Object(vars[i].m_value, PProcess::Current().GetVersion(true));
 			found = true;
@@ -584,7 +557,6 @@ void WindowsSNMPAgent::Run()
 				PTRACE(1, "SNMP\tSetNamedPipeHandleState failed. GLE=" << GetLastError());
 			}
 
-			PTRACE(3, "JW connected SNMP extension DLL pipes");
 			// read server request and respond, then disconnect
 			char buffer[BUFSIZE];
 			DWORD bytesRead;
