@@ -169,12 +169,6 @@ bool YaSocket::Read(void * buf, int sz)
 	return lastReadCount > 0;
 }
 
-bool YaSocket::ReadBlock(void * buf, int len)
-{
-	// lazy implementation, but it is enough for us...
-	return Read(buf, len) && lastReadCount == len;
-}
-
 bool YaSocket::CanRead(long timeout) const
 {
 	const int h = os_handle;
@@ -281,7 +275,7 @@ bool YaSocket::ConvertOSError(int libReturnValue, PSocket::ErrorGroup group)
 		lastErrorNumber[group] = errno;
 		return false;
 	}
-	return PSocket::ConvertOSError(libReturnValue, lastErrorCode[group], lastErrorNumber[group]);
+	return PChannel::ConvertOSError(libReturnValue, lastErrorCode[group], lastErrorNumber[group]);
 }
 
 bool YaSocket::SetNonBlockingMode()
@@ -541,6 +535,7 @@ int YaTCPSocket::os_send(const void * buf, int sz)
 // class YaUDPSocket
 YaUDPSocket::YaUDPSocket(WORD port, int iAddressFamily)
 {
+	memset(&recvaddr, 0, sizeof(recvaddr));
 	((struct sockaddr*)&sendaddr)->sa_family = iAddressFamily;
 	((struct sockaddr_in*)&sendaddr)->sin_port = htons(port);
 }
@@ -841,7 +836,7 @@ bool USocket::Flush()
 	bool result = true;
 	PWaitAndSignal lock(writeMutex);
 	while (result && qsize > 0) {
-		PBYTEArray* const pdata = PopQueuedPacket();
+		PBYTEArray * const pdata = PopQueuedPacket();
 		if (pdata) {
 			result = InternalWriteData(*pdata, pdata->GetSize());
 			unsigned bytesSent = self->GetLastWriteCount();
@@ -857,16 +852,9 @@ bool USocket::WriteData(const BYTE * buf, int len)
 {
 	if (!IsSocketOpen())
 		return false;
-		
+
 	int remaining = len;
-	if (qsize == 0 && 
-#ifdef hasNoMutexWillBlock
-		writeMutex.Try()
-#else
-		!writeMutex.WillBlock()
-#endif
-		) {
-		PWaitAndSignal lock(writeMutex);
+	if (qsize == 0 && writeMutex.Wait(0)) {
 		while (remaining > 0) {
 			int sendnow = remaining > MAX_SOCKET_CHUNK ? MAX_SOCKET_CHUNK : remaining;
 			if (!InternalWriteData(buf, sendnow)) {
@@ -881,6 +869,7 @@ bool USocket::WriteData(const BYTE * buf, int len)
 					PThread::Sleep(SOCKET_CHUNK_PAUSE);
 			}
 		}
+		writeMutex.Signal();
 		if (remaining == 0)
 			return true;
 	}
@@ -1131,19 +1120,31 @@ bool TCPServer::CloseListener(TCPListenSocket * socket)
 
 void TCPServer::ReadSocket(IPSocket * socket)
 {
+	// don't accept new calls when shutdown is already in progress
+	bool shutdown = false;
 #ifdef hasNoMutexWillBlock
+	// TODO: this might be a performance bottleneck on high load
 	if (!ShutdownMutex.Wait(0)) {
+		shutdown = true;
+	} else {
+		ShutdownMutex.Signal();	// release mutex, we were just checking for shutdown
+	}
 #else
-	if (ShutdownMutex.WillBlock()) {
+	if (ShutdownMutex.WillBlock())
+		shutdown = true;
 #endif
+
+	if (shutdown) {
 		PTRACE(4, GetName() << "\tShutdown: Rejecting call on " << socket->GetName());
 		int rej = ::accept(socket->GetHandle(), NULL, NULL);
-		::shutdown(rej, SHUT_RDWR );
+		if (rej >= 0) {
+			::shutdown(rej, SHUT_RDWR);
 #if defined(_WIN32)
-		::closesocket(rej);
+			::closesocket(rej);
 #else
-		::close(rej);
+			::close(rej);
 #endif
+		}
 		return;
 	}
 
@@ -1159,12 +1160,14 @@ void TCPServer::ReadSocket(IPSocket * socket)
 			// reject call
 			PTRACE(1, GetName() << "\tRate limit reached (max " << cps_limit << " cps) - rejecting call on " << socket->GetName());
 			int rej = ::accept(socket->GetHandle(), NULL, NULL);
-			::shutdown(rej, 2 /* SHUT_RDWR */ );
+			if (rej >= 0) {
+				::shutdown(rej, SHUT_RDWR);
 #if defined(_WIN32)
-			::closesocket(rej);
+				::closesocket(rej);
 #else
-			::close(rej);
+				::close(rej);
 #endif
+			}
 			return;
 		}
 		// add accepted calls to stats list
